# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import argparse
import sys

import paramiko

from oslo_log import log

LOG = log.getLogger(__name__)


class Tobiko():

    def __init__(self):
        self.parser = self.get_parser()
        self.args = (self.parser).parse_args()
        self.ssh = paramiko.SSHClient()
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    def get_parser(self):
        parser = argparse.ArgumentParser(add_help=True)
        parser.add_argument(
            '--host',
            help="The name of the host where your cloud is deployed.\n")
        parser.add_argument(
            '--key', '-k',
            help="They SSH key to use to connect the host.")
        return parser

    def verify_connection(self):
        """Verifies it's able to connect the host provided by the user."""
        try:
            self.ssh.connect(self.args.host)
        except paramiko.ssh_exception.AuthenticationException:
            LOG.error("Unable to connect %r", self.args.host)


def main():
    """Run CLI main entry."""
    tobiko = Tobiko()
    tobiko.verify_connection()
    # run.discover_environment()


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import abc

from oslo_log import log

import tobiko
from tobiko.openstack import keystone


LOG = log.getLogger(__name__)


class OpenstackClientFixture(tobiko.SharedFixture):

    client = None
    session = None
    session_fixture = None

    def __init__(self, session=None):
        super(OpenstackClientFixture, self).__init__()
        if session:
            if tobiko.is_fixture(session):
                self.session_fixture = session
            else:
                self.session = session

    def setup_fixture(self):
        self.setup_session()
        self.setup_client()

    def setup_session(self):
        session_fixture = self.session_fixture
        if session_fixture:
            self.session = tobiko.setup_fixture(session_fixture).session
        elif not self.session:
            self.session = keystone.get_keystone_session()

    def setup_client(self):
        self.client = self.init_client(session=self.session)

    @abc.abstractmethod
    def init_client(self, session):
        raise NotImplementedError


class OpenstackClientManager(object):

    def __init__(self, init_client=None):
        self._clients = {}
        self.init_client = init_client

    def get_client(self, session=None, shared=True, init_client=None):
        if shared:
            if session and tobiko.is_fixture(session):
                key = tobiko.get_fixture_name(session)
            else:
                key = session

            client = self._clients.get(key)
            if client:
                return client

        session = session or keystone.get_keystone_session()
        init_client = init_client or self.init_client
        assert callable(init_client)
        LOG.debug('Initialize OpenStack client: %r(session=%r)',
                  init_client, session)
        client = init_client(session=session)

        if shared:
            self._clients[key] = client
        return client



# Copyright (c) 2018 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os
import logging
import argparse

from oslo_log import log

from tobiko.common.managers import ansible
from tobiko import config


LOG = log.getLogger(__name__)


class TobikoCMD(object):
    """Manages different command line utilities."""

    def __init__(self):
        config.CONF.tobiko.use_stderr = True
        log.setup(config.CONF.tobiko, 'tobiko')
        self.parser = self.get_parser()
        self.args = (self.parser).parse_args()

        curr_dir = os.path.dirname(__file__)
        self.playbooks_dir = os.path.join(curr_dir,
                                          "../tests/scenario/playbooks")
        self.ansibleManager = ansible.AnsibleManager(self.playbooks_dir)

    def get_parser(self):
        parser = argparse.ArgumentParser(add_help=True)
        parser.add_argument('--verbose', '-v', action='count',
                            help='Make the output more verbose, incremental.')
        parser.add_argument('--quiet', '-q', action='count',
                            help='Make the output less verbose, incremental.')
        return parser

    def set_stream_handler_logging_level(self):
        num_quiet = self.args.quiet or 0
        num_verb = self.args.verbose or 0
        level = logging.WARNING - (num_verb * 10) + (num_quiet * 10)
        root_logger = logging.getLogger()
        for handler in root_logger.handlers:
            if isinstance(handler, logging.StreamHandler):
                handler.setLevel(level)



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from oslo_config import cfg


def register_tobiko_options(conf):
    conf.register_opts(
        group=cfg.OptGroup('nova'),
        opts=[cfg.StrOpt('image',
                         help="Default image for new server instances"),
              cfg.StrOpt('flavor',
                         help="Default flavor for new server instances"),
              cfg.StrOpt('key_file', default='~/.ssh/id_rsa',
                         help="Default SSH key to login to server instances"),
              cfg.StrOpt('username', default='cirros',
                         help="Default username to login to server instances"),
              cfg.StrOpt('password', default='gocubsgo',
                         help="Default password to login to server instances"),
              ])



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import sys

from oslo_log import log

import jsonschema
import os_faults

from tobiko.fault.config import FaultConfig

LOG = log.getLogger(__name__)


class FaultExecutor(object):
    """Responsible for executing faults."""

    def __init__(self, conf_file=None, cloud=None):
        self.config = FaultConfig(conf_file=conf_file)
        self.cloud = cloud

    def connect(self):
        """Connect to the cloud using os-faults."""
        try:
            self.cloud = os_faults.connect(
                config_filename=self.config.conf_file)
            self.cloud.verify()
        except os_faults.ansible.executor.AnsibleExecutionUnreachable:
            LOG.warning("Couldn't verify connectivity to the"
                        " cloud with os-faults configuration")
        except jsonschema.exceptions.ValidationError:
            LOG.error("Wrong os-fault configuration format. Exiting...")
            sys.exit(2)

    def execute(self, fault):
        """Executes given fault using os-faults human API."""
        LOG.info("Using %s" % self.config.conf_file)
        self.connect()
        os_faults.human_api(self.cloud, fault)



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import subprocess

import six


def shell_command(command):
    if isinstance(command, ShellCommand):
        return command
    elif isinstance(command, six.string_types):
        return ShellCommand(command.split())
    elif command:
        return ShellCommand(str(a) for a in command)
    else:
        return ShellCommand()


class ShellCommand(tuple):

    def __repr__(self):
        return "ShellCommand([{!s}])".format(', '.join(self))

    def __str__(self):
        return subprocess.list2cmdline(self)

    def __add__(self, other):
        other = shell_command(other)
        return shell_command(tuple(self) + other)



# Copyright (c) 2019 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

from tobiko.tests import unit
from tobiko import config


CONF = config.CONF


class HttpProxyFixtureTest(unit.TobikoUnitTest):

    MY_HTTP_PROXY = 'http://my-server:8080'
    MY_NO_PROXY = '127.0.0.1'

    def setUp(self):
        super(HttpProxyFixtureTest, self).setUp()
        self.patch(os, 'environ', {})
        self.patch(CONF.tobiko, 'http',
                   http_proxy=None, https_proxy=None, no_proxy=None)

    def test_init(self):
        fixture = config.HttpProxyFixture()
        self.assertIsNone(fixture.http_proxy)
        self.assertEqual({}, os.environ)

    def test_setup(self):
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertIsNone(fixture.http_proxy)
        self.assertEqual({}, os.environ)

    def test_setup_from_environ_http_proxy(self):
        os.environ['http_proxy'] = self.MY_HTTP_PROXY
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertEqual({'http_proxy': self.MY_HTTP_PROXY}, os.environ)
        self.assertEqual(self.MY_HTTP_PROXY, fixture.http_proxy)

    def test_setup_from_environ_https_proxy(self):
        os.environ['https_proxy'] = self.MY_HTTP_PROXY
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertEqual({'https_proxy': self.MY_HTTP_PROXY}, os.environ)
        self.assertEqual(self.MY_HTTP_PROXY, fixture.https_proxy)

    def test_setup_from_environ_no_proxy(self):
        os.environ['no_proxy'] = self.MY_NO_PROXY
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertEqual({'no_proxy': self.MY_NO_PROXY}, os.environ)
        self.assertEqual(self.MY_NO_PROXY, fixture.no_proxy)

    def test_setup_from_tobiko_conf_http_proxy(self):
        self.patch(CONF.tobiko.http, 'http_proxy', self.MY_HTTP_PROXY)
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertEqual(self.MY_HTTP_PROXY, fixture.http_proxy)
        self.assertEqual({'http_proxy': self.MY_HTTP_PROXY}, os.environ)

    def test_setup_from_tobiko_conf_https_proxy(self):
        self.patch(CONF.tobiko.http, 'https_proxy', self.MY_HTTP_PROXY)
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertEqual(self.MY_HTTP_PROXY, fixture.https_proxy)
        self.assertEqual({'https_proxy': self.MY_HTTP_PROXY}, os.environ)

    def test_setup_from_tobiko_conf_no_proxy(self):
        self.patch(CONF.tobiko.http, 'http_proxy', self.MY_HTTP_PROXY)
        self.patch(CONF.tobiko.http, 'no_proxy', self.MY_NO_PROXY)
        fixture = config.HttpProxyFixture()

        fixture.setUp()

        self.assertEqual(self.MY_NO_PROXY, fixture.no_proxy)
        self.assertEqual(self.MY_HTTP_PROXY, fixture.http_proxy)
        self.assertEqual({'no_proxy': self.MY_NO_PROXY,
                          'http_proxy': self.MY_HTTP_PROXY}, os.environ)



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import collections
import time

from heatclient import exc
from oslo_log import log

import tobiko
from tobiko.openstack.heat import _client
from tobiko.openstack.heat import _template


LOG = log.getLogger(__name__)

# Status
INIT_IN_PROGRESS = 'INIT_IN_PROGRESS'
INIT_COMPLETE = 'INIT_COMPLETE'
INIT_IN_PROGRESS = 'INIT_FAILED'
CREATE_IN_PROGRESS = 'CREATE_IN_PROGRESS'
CREATE_COMPLETE = 'CREATE_COMPLETE'
CREATE_FAILED = 'CREATE_FAILED'
DELETE_IN_PROGRESS = 'DELETE_IN_PROGRESS'
DELETE_COMPLETE = 'DELETE_COMPLETE'
DELETE_FAILED = 'DELETE_FAILED'


TEMPLATE_FILE_SUFFIX = '.yaml'


class HeatStackFixture(tobiko.SharedFixture):
    """Manages Heat stacks."""

    client = None
    client_fixture = None
    retry_create_stack = 1
    wait_interval = 5
    stack_name = None
    template = None
    template_fixture = None
    parameters = None
    stack = None

    def __init__(self, stack_name=None, template=None, parameters=None,
                 wait_interval=None, client=None):
        super(HeatStackFixture, self).__init__()
        self.stack_name = stack_name = (stack_name or
                                        self.stack_name or
                                        self.fixture_name)

        template = template or self.template
        if tobiko.is_fixture(template):
            self.template = None
            self.template_fixture = template
        elif isinstance(template, collections.Mapping):
            self.template = _template.HeatTemplate.from_dict(template)
            self.template_fixture = None
        elif template:
            msg = "Invalid type for template parameter: {!r}".format(template)
            raise TypeError(msg)
        self._parameters = parameters

        if tobiko.is_fixture(client):
            self.client_fixture = client
        elif client:
            self.client = client

        if wait_interval:
            self.wait_interval = wait_interval

    def setup_fixture(self):
        self.setup_template()
        self.setup_parameters()
        self.setup_client()
        self.setup_stack()

    def setup_template(self):
        template_fixture = self.template_fixture
        if template_fixture:
            self.template = tobiko.setup_fixture(template_fixture).template
        elif not self.template:
            template_name = self.stack_name.rsplit('.')[-1]
            self.template = _template.get_heat_template(
                template_file=(template_name + TEMPLATE_FILE_SUFFIX),
                template_dirs=[tobiko.get_fixture_dir(self)])

    def setup_parameters(self):
        self.parameters = {}
        # Merge all parameters dictionaries in the class hierarchy
        for cls in reversed(type(self).__mro__):
            parameters = cls.__dict__.get('parameters')
            if parameters:
                self.parameters.update(parameters)

        if self._parameters:
            self.parameters.update(self._parameters)

        # Add template's missing stack parameters
        if 'parameters' in self.template.template:
            for name in self.template.template['parameters']:
                if name not in self.parameters:
                    value = getattr(self, name, None)
                    if value is not None:
                        self.parameters[name] = value

    def setup_client(self):
        client_fixture = self.client_fixture
        if client_fixture:
            self.client = tobiko.setup_fixture(client_fixture).client
        elif not self.client:
            self.client = _client.get_heat_client()

    def setup_stack(self):
        self.create_stack()

    def create_stack(self, retry=None):
        """Creates stack based on passed parameters."""
        created_stack_ids = set()
        retry = retry or self.retry_create_stack or 1
        while True:
            stack = self.wait_for_stack_status(
                expected_status={CREATE_COMPLETE, CREATE_FAILED,
                                 CREATE_IN_PROGRESS, DELETE_COMPLETE,
                                 DELETE_FAILED})
            stack_status = getattr(stack, 'stack_status', DELETE_COMPLETE)
            expected_status = {CREATE_COMPLETE, CREATE_IN_PROGRESS}
            if stack_status in expected_status:
                LOG.debug('Stack created: %r (id=%r)', self.stack_name,
                          stack.id)
                for stack_id in created_stack_ids:
                    if self.stack.id != stack_id:
                        LOG.warning("Concurrent stack creation: delete "
                                    "duplicated stack is %r (id=%r).",
                                    self.stack_name, stack_id)
                        self.delete_stack(stack_id)

                return stack

            if not retry:
                status_reason = getattr(stack, 'stack_status_reason', None)
                raise HeatStackCreationFailed(name=self.stack_name,
                                              observed=stack_status,
                                              expected=expected_status,
                                              status_reason=status_reason)

            retry -= 1
            if stack_status.endswith('_FAILED'):
                LOG.debug('Delete existing failed stack: %r (id=%r)',
                          self.stack_name, stack.id)
                self.delete_stack()
                stack = self.wait_for_stack_status(
                    expected_status={DELETE_COMPLETE})

            self.stack = self._outputs = None
            try:
                LOG.debug('Creating stack %r (re-tries left %d)...',
                          self.stack_name, retry)
                stack_id = self.client.stacks.create(
                    stack_name=self.stack_name,
                    template=self.template.yaml,
                    parameters=self.parameters)['stack']['id']
            except exc.HTTPConflict:
                LOG.debug('Stack %r already exists.', self.stack_name)
            else:
                created_stack_ids.add(stack_id)
                LOG.debug('Creating stack %r (id=%r)...', self.stack_name,
                          stack_id)

    def cleanup_fixture(self):
        self.setup_client()
        self.cleanup_stack()

    def cleanup_stack(self):
        self.delete_stack()

    def delete_stack(self, stack_id=None):
        """Deletes stack."""
        if not stack_id:
            stack_id = self.stack_id
            self.stack = self._outputs = None
        try:
            self.client.stacks.delete(stack_id)
        except exc.NotFound:
            LOG.debug('Stack already deleted: %r (id=%r)', self.stack_name,
                      stack_id)
        else:
            LOG.debug('Deleting stack %r (id=%r)...', self.stack_name,
                      stack_id)

    @property
    def stack_id(self):
        stack = self.stack
        if stack:
            return stack.id
        else:
            return self.stack_name

    def get_stack(self, resolve_outputs=False):
        """Returns stack ID."""
        try:
            self.stack = stack = self.client.stacks.get(
                self.stack_name, resolve_outputs=resolve_outputs)
        except exc.HTTPNotFound:
            self.stack = stack = None
        finally:
            self._outputs = None
        return stack

    def wait_for_create_complete(self, check=True):
        return self.wait_for_stack_status(expected_status={CREATE_COMPLETE},
                                          check=check)

    def wait_for_delete_complete(self, check=True):
        return self.wait_for_stack_status(expected_status={DELETE_COMPLETE},
                                          check=check)

    def wait_for_stack_status(self, expected_status, check=True):
        """Waits for the stack to reach the given status."""
        stack = self.stack or self.get_stack()
        while (stack and stack.stack_status.endswith('_IN_PROGRESS') and
               stack.stack_status not in expected_status):
            LOG.debug("Waiting for %r (id=%r) stack status "
                      "(observed=%r, expected=%r)", self.stack_name,
                      stack.id, stack.stack_status, expected_status)
            time.sleep(self.wait_interval)
            stack = self.get_stack()

        if check:
            if stack is None:
                if DELETE_COMPLETE not in expected_status:
                    raise HeatStackNotFound(name=self.stack_name)
            else:
                check_stack_status(stack, expected_status)
        return stack

    _outputs = None

    @property
    def outputs(self):
        return self.wait_for_outputs()

    def get_outputs(self):
        stack = self.stack
        if not hasattr(stack, 'outputs'):
            stack = self.get_stack(resolve_outputs=True)
        check_stack_status(stack, {CREATE_COMPLETE})
        self._outputs = outputs = HeatStackOutputs(
            stack_name=self.stack_name,
            outputs={output['output_key']: output['output_value']
                     for output in stack.outputs})
        return outputs

    def wait_for_outputs(self):
        if self._outputs:
            return self._outputs
        else:
            self.wait_for_create_complete()
            return self.get_outputs()


class HeatStackOutputs(object):

    def __init__(self, stack_name, outputs):
        self.stack_name = stack_name
        self.outputs = outputs

    def __getattr__(self, name):
        try:
            return self.outputs[name]
        except KeyError:
            raise InvalidHeatStackOutputKey(name=self.stack_name,
                                            key=name)


def check_stack_status(stack, expected):
    observed = stack.stack_status
    if observed not in expected:
        if observed == CREATE_FAILED:
            error_class = HeatStackCreationFailed
        elif observed == DELETE_FAILED:
            error_class = HeatStackDeletionFailed
        else:
            error_class = InvalidHeatStackStatus
        raise error_class(name=stack.stack_name,
                          observed=observed,
                          expected=expected,
                          status_reason=stack.stack_status_reason)


class InvalidHeatStackOutputKey(tobiko.TobikoException):
    message = "output key {key!r} not found in stack {name!r}"


class HeatStackNotFound(tobiko.TobikoException):
    message = "stack {name!r} not found"


class InvalidHeatStackStatus(tobiko.TobikoException):
    message = ("stack {name!r} status {observed!r} not in {expected!r}\n"
               "{status_reason!s}")


class HeatStackCreationFailed(InvalidHeatStackStatus):
    pass


class HeatStackDeletionFailed(InvalidHeatStackStatus):
    pass



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

import os_faults
import yaml

from oslo_log import log

LOG = log.getLogger(__name__)

try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError  # pylint: disable=redefined-builtin

CONF_DIR = os.path.expanduser('~')
CONF_FILE = 'os-faults.yaml'


class FaultManager():
    """Manages faults."""

    def __init__(self, test_file):
        self.faults_dir = os.path.join(os.path.dirname(test_file), 'faults')
        faults_f_name = os.path.splitext(os.path.basename(test_file))[0]
        self.faults_file = os.path.join(self.faults_dir, faults_f_name)
        fault_config_f = os.path.join(CONF_DIR, CONF_FILE)
        try:
            self.cloud = os_faults.connect(config_filename=fault_config_f)
            self.cloud.verify()
            self.scenarios = self.get_scenarios()
        except os_faults.ansible.executor.AnsibleExecutionUnreachable:
            LOG.warning("Couldn't verify connectivity to the"
                        " cloud with os-faults configuration")
            self.scenarios = None
        except FileNotFoundError:
            LOG.warning("Couldn't find os-faults configuration file")
            self.scenarios = None

    def get_scenarios(self):
        """Returns list of scenarios based on defined faults.

        A scenario composed out of scenario name and the fault to execute.
        """
        scenarios = []
        with open(self.faults_file, 'r') as stream:
            faults_yaml = yaml.load(stream)
            for fault in faults_yaml:
                scenarios.append((fault['name'], dict(fault=fault['action'])))
            return scenarios

    def run_fault(self, fault):
        """Executes given fault."""
        if self.scenarios:
            os_faults.human_api(self.cloud, fault)
        else:
            LOG.debug("Skipped fault: '{}' since".format(fault),
                      " scenarios are not defined.")



# Copyright (c) 2018 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import sys

from tobiko.cmd import base
from tobiko.tests.unit import openstack


class TobikoCMDTest(openstack.OpenstackTest):

    command_name = 'tobiko-base'
    command_class = base.TobikoCMD

    def test_init(self, argv=None):
        self.patch_argv(argv=argv)
        return self.command_class()

    def patch_argv(self, argv=None):
        return self.patch(sys, 'argv', [self.command_name] + (argv or []))



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os
import socket
import time

import paramiko
from oslo_log import log

import tobiko
from tobiko.shell.ssh import _config
from tobiko.shell.ssh import _command


LOG = log.getLogger(__name__)


SSH_CONNECT_PARAMETERS = {
    #: The server to connect to
    'hostname': str,

    #: The server port to connect to
    'port': int,

    #: The username to authenticate as (defaults to the current local username)
    'username': str,

    #: Used for password authentication; is also used for private key
    #: decryption if passphrase is not given
    'password': str,

    #: Used for decrypting private keys
    'passphrase': str,

    #: The filename, or list of filenames, of optional private key(s) and/or
    #: certs to try for authentication
    'key_filename': str,

    #: An optional timeout (in seconds) for the TCP connect
    'timeout': float,

    #: Set to False to disable connecting to the SSH agent
    'allow_agent': bool,

    #: Set to False to disable searching for discoverable private key files in
    #: ~/.ssh/
    'look_for_keys': bool,

    #: Set to True to turn on compression
    'compress': bool,

    #: True if you want to use GSS-API authentication
    'gss_auth': bool,

    #: Perform GSS-API Key Exchange and user authentication
    'gss_kex': bool,

    #: Delegate GSS-API client credentials or not
    'gss_deleg_creds': bool,

    #: The targets name in the kerberos database. default: hostname
    'gss_host': str,

    #: Indicates whether or not the DNS is trusted to securely canonicalize the
    #: name of the host being connected to (default True).
    'gss_trust_dns': bool,

    #: An optional timeout (in seconds) to wait for the SSH banner to be
    #: presented.
    'banner_timeout': float,

    #: An optional timeout (in seconds) to wait for an authentication response
    'auth_timeout': float,

    #: Number of connection attempts to be tried before timeout
    'connection_attempts': int,

    #: Minimum amount of time to wait between two connection attempts
    'connection_interval': float,

    #: Command to be executed to open proxy sock
    'proxy_command': str,
}


class SSHConnectFailure(tobiko.TobikoException):
    message = "Failed to login to {login}\n{cause}"


class SSHClientFixture(tobiko.SharedFixture):

    host = None
    client = None

    default = tobiko.required_setup_fixture(_config.SSHDefaultConfigFixture)
    config_files = None
    host_config = None

    proxy_client = None
    proxy_sock = None
    connect_parameters = None

    def __init__(self, host=None, proxy_client=None, host_config=None,
                 config_files=None, **connect_parameters):
        super(SSHClientFixture, self).__init__()
        if host:
            self.host = host
        if proxy_client:
            self.proxy_client = proxy_client
        if host_config:
            self.host_config = host_config
        if config_files:
            self.config_files = config_files
        invalid_parameters = sorted([name
                                     for name in connect_parameters
                                     if name not in SSH_CONNECT_PARAMETERS])
        if invalid_parameters:
            message = "Invalid SSH connection parameters: {!s}".format(
                ', '.join(invalid_parameters))
            raise ValueError(message)
        self._connect_parameters = connect_parameters

    def setup_fixture(self):
        self.setup_host_config()
        self.setup_connect_parameters()
        self.setup_ssh_client()

    def setup_host_config(self):
        if not self.host_config:
            self.host_config = _config.ssh_host_config(
                host=self.host, config_files=self.config_files)

    def setup_connect_parameters(self):
        """Fill connect parameters dict

        Get parameters values from below sources:
        - parameters passed to class constructor
        - parameters got from ~/.ssh/config and tobiko.conf
        - parameters got from fixture object attributes
        """

        # Get default parameter values from self object
        self.connect_parameters = parameters = items_from_object(
            schema=SSH_CONNECT_PARAMETERS, obj=self)
        LOG.debug('Default parameters for host %r:\n%r', self.host,
                  parameters)

        # Override parameters from host configuration files
        parameters.update(
            items_from_mapping(schema=SSH_CONNECT_PARAMETERS,
                               mapping=self.host_config.connect_parameters))
        LOG.debug('Configured connect parameters for host %r:\n%r',
                  self.host, parameters)

        # Override parameters with __init__ parameters
        parameters.update(
            items_from_mapping(schema=SSH_CONNECT_PARAMETERS,
                               mapping=self._connect_parameters))
        LOG.debug('Resulting connect parameters for host %r:\n%r', self.host,
                  parameters)

        # Validate hostname
        hostname = parameters.get('hostname')
        if not hostname:
            message = "Invalid hostname: {!r}".format(hostname)
            raise ValueError(message)

        # Expand key_filename
        key_filename = parameters.get('key_filename')
        if key_filename:
            key_filename = os.path.expanduser(key_filename)
            if not os.path.exists(key_filename):
                message = "key_filename {!r} doesn't exist".format(hostname)
                raise ValueError(message)
            parameters['key_filename'] = key_filename

        # Validate connection timeout
        timeout = parameters.get('timeout')
        if not timeout or timeout < 0.:
            message = "Invalid timeout: {!r}".format(timeout)
            raise ValueError(message)

        # Validate connection attempts
        connection_attempts = parameters.get('connection_attempts')
        if not connection_attempts or connection_attempts < 0:
            message = "Invalid connection attempts: {!r}".format(
                connection_attempts)
            raise ValueError(message)

        # Validate connection attempts
        connection_interval = parameters.get('connection_interval')
        if not connection_interval or connection_interval < 0.:
            message = "Invalid connection interval: {!r}".format(
                connection_interval)
            raise ValueError(message)

        # Validate connection port
        port = parameters.get('port')
        if not port or port < 1 or port > 65535:
            message = "Invalid port: {!r}".format(port)
            raise ValueError(message)

    def setup_ssh_client(self):
        self.client, self.proxy_sock = ssh_connect(
            proxy_client=self.proxy_client, **self.connect_parameters)
        self.addCleanup(self.client.close)
        if self.proxy_sock:
            self.addCleanup(self.proxy_sock.close)

    def connect(self):
        return tobiko.setup_fixture(self).client


def items_from_mapping(schema, mapping):
    return ((key, init(mapping.get(key)))
            for key, init in schema.items()
            if mapping.get(key) is not None)


def items_from_object(schema, obj):
    return {key: init(getattr(obj, key, None))
            for key, init in schema.items()
            if getattr(obj, key, None) is not None}


UNDEFINED_CLIENT = 'UNDEFINED_CLIENT'


class SSHClientManager(object):

    default = tobiko.required_setup_fixture(_config.SSHDefaultConfigFixture)

    def __init__(self):
        self.clients = {}

    def get_client(self, host, username=None, port=None, proxy_jump=None,
                   host_config=None, config_files=None, **connect_parameters):
        host_config = host_config or _config.ssh_host_config(
            host=host, config_files=config_files)
        hostname = host_config.hostname
        port = port or host_config.port
        username = username or host_config.username
        host_key = hostname, port, username, proxy_jump
        client = self.clients.get(host_key, UNDEFINED_CLIENT)
        if client is UNDEFINED_CLIENT:
            # Put a placeholder client to avoid infinite recursive lookup
            self.clients[host_key] = None
            proxy_client = self.get_proxy_client(host=host,
                                                 config_files=config_files)
            self.clients[host_key] = client = SSHClientFixture(
                host=host, hostname=hostname, port=port, username=username,
                proxy_client=proxy_client, **connect_parameters)
        return client

    def get_proxy_client(self, host=None, host_config=None,
                         config_files=None):
        host_config = host_config or _config.ssh_host_config(
            host=host, config_files=config_files)
        proxy_host = host_config.proxy_jump
        return proxy_host and self.get_client(proxy_host) or None


CLIENTS = SSHClientManager()


def ssh_client(host, port=None, username=None, proxy_jump=None,
               host_config=None, config_files=None, manager=None,
               **connect_parameters):
    manager = manager or CLIENTS
    return manager.get_client(host=host, port=port, username=username,
                              proxy_jump=proxy_jump, host_config=host_config,
                              config_files=config_files,
                              **connect_parameters)


def ssh_connect(hostname, username=None, port=None, connection_interval=None,
                connection_attempts=None, proxy_command=None,
                proxy_client=None, **parameters):
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    client.load_system_host_keys()

    login = _command.ssh_login(hostname=hostname, username=username, port=port)
    attempts = connection_attempts or 1
    interval = connection_interval or 5.
    for attempt in range(1, attempts + 1):
        LOG.debug("Logging in to %r (%r)... attempt %d out of %d",
                  login, parameters, attempt, attempts)
        start_time = time.time()
        proxy_sock = ssh_proxy_sock(hostname=hostname,
                                    port=port,
                                    command=proxy_command,
                                    client=proxy_client)
        try:
            client.connect(hostname=hostname,
                           username=username,
                           port=port,
                           sock=proxy_sock,
                           **parameters)
        except (EOFError, socket.error, socket.timeout,
                paramiko.SSHException) as ex:
            if attempt >= attempts:
                raise

            LOG.debug("Error logging in to %r: \n(%s)", login, ex)
            sleep_time = start_time + interval - time.time()
            if sleep_time > 0.:
                LOG.debug("Retrying connecting to %r in %d seconds...", login,
                          sleep_time)
                time.sleep(sleep_time)

        else:
            LOG.info("Successfully logged it to %s", login)
            return client, proxy_sock


def ssh_proxy_sock(hostname, port=None, command=None, client=None):
    if client:
        # I need a command to execute with proxy client
        command = command or 'nc {hostname!r} {port!r}'
    elif not command:
        # Proxy sock is not required
        return None

    # Apply connect parameters to proxy command
    command = command.format(hostname=hostname, port=(port or 22))
    if client:
        if isinstance(client, SSHClientFixture):
            # Connect to proxy server
            client = client.connect()
        elif not isinstance(client, paramiko.SSHClient):
            message = "Object {!r} is not an SSHClient".format(client)
            raise TypeError(message)

        # Open proxy channel
        LOG.debug("Execute proxy command with proxy client %r: %r",
                  client, command)
        sock = client.get_transport().open_session()
        sock.exec_command(command)
    else:
        LOG.debug("Execute proxy command on local host: %r", command)
        sock = paramiko.ProxyCommand(command)

    return sock


def ssh_proxy_client(manager=None, host=None, host_config=None,
                     config_files=None):
    manager = manager or CLIENTS
    return manager.get_proxy_client(host=host,
                                    host_config=host_config,
                                    config_files=config_files)



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import tobiko
from tobiko.tests import unit


class SomeException(tobiko.TobikoException):
    message = "message formatted with {b} and {a}"


class TestException(unit.TobikoUnitTest):

    def test_init(self, message=None, **properties):
        ex = SomeException(message, **properties)
        expected_str = message or SomeException.message
        if properties:
            expected_str = expected_str.format(**properties)
        self.assertEqual(expected_str, str(ex))
        for k, v in properties.items():
            self.assertEqual(v, getattr(ex, k))

    def test_init_with_properties(self):
        self.test_init(a='a', b='b')

    def test_init_with_message(self):
        self.test_init('{other} message', other='another')

    def test_raise(self):
        def _raise_my_exception(**kwargs):
            raise SomeException(**kwargs)
        ex = self.assertRaises(SomeException, _raise_my_exception, b=1, a=2)
        self.assertEqual('message formatted with 1 and 2', str(ex))

    def test_repr(self):
        ex = SomeException('some reason')
        self.assertEqual("SomeException('some reason')", repr(ex))

    def test_get_invalid_property(self):
        ex = SomeException(a='1', b='2')

        def _get_invalid_property():
            return ex.invalid_attribute_name

        ex = self.assertRaises(AttributeError, _get_invalid_property)
        self.assertEqual(
            "SomeException('message formatted with 2 and 1') object has no "
            "attribute 'invalid_attribute_name'", str(ex))

    def test_docstring_example(self):

        class MyException(tobiko.TobikoException):
            message = "This exception occurred because of {reason}"

        try:
            raise MyException(reason="something went wrong")
        except MyException as ex:
            self.assertEqual(
                'This exception occurred because of something went wrong',
                str(ex))
            self.assertEqual('something went wrong', ex.reason)



# Copyright (c) 2019 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

from tobiko import config
from tobiko.openstack import heat
from tobiko.openstack import neutron


CONF = config.CONF


TEMPLATE_DIRS = [os.path.join(os.path.dirname(__file__), 'templates')]


def heat_template_file(template_file):
    """Fixture to load template files from templates directory

    Return fixtures to loads templates from
    'tobiko/tests/scenario/neutron/templates' directory
    """
    return heat.heat_template_file(template_file=template_file,
                                   template_dirs=TEMPLATE_DIRS)


class InternalNetworkFixture(heat.HeatStackFixture):
    """Heat stack for creating internal network with a router to external

    """

    #: Heat template file
    template = heat_template_file('internal_network.yaml')

    #: Floating IP network where the Neutron floating IP is created
    floating_network = CONF.tobiko.neutron.floating_network

    #: Disable port security by default for new network ports
    port_security_enabled = False

    #: whenever has net-mtu networking extension
    @property
    def has_net_mtu(self):
        return neutron.has_networking_extensions('net-mtu')

    #: Value for maximum transfer unit on the internal network
    mtu = None

    def setup_parameters(self):
        """Setup template parameters

        """
        super(InternalNetworkFixture, self).setup_parameters()
        if self.port_security_enabled or neutron.has_networking_extensions(
                'port-security'):
            self.setup_port_security()
        if self.mtu:
            self.setup_net_mtu_writable()

    @neutron.skip_if_missing_networking_extensions('port-security')
    def setup_port_security(self):
        """Setup default port security value for internal network ports

        """
        self.parameters.update(
            port_security_enabled=self.port_security_enabled)

    @neutron.skip_if_missing_networking_extensions('net-mtu-writable')
    def setup_net_mtu_writable(self):
        """Setup maximum transfer unit size for internal network

        """
        self.parameters.setdefault('value_specs', {}).update(mtu=self.mtu)


class InternalNetworkFixtureWithPortSecurity(InternalNetworkFixture):
    """Heat stack for creating internal network with port security

    """
    #: Enable port security by default for new network ports
    port_security_enabled = True


@neutron.skip_if_missing_networking_extensions('net-mtu-writable')
class InternalNetworkWithNetMtuWritableFixture(InternalNetworkFixture):
    """Heat stack for creating internal network with custom mtu parameter

    """
    #: Force value for internal network maximum transfer unit
    mtu = 1000


@neutron.skip_if_missing_networking_extensions('security-group')
class SecurityGroupsFixture(heat.HeatStackFixture):
    """Heat stack with some security groups

    """
    #: Heat template file
    template = heat_template_file('security_groups.yaml')



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

from tobiko.openstack import heat
from tobiko.tests.unit import openstack


class GetHeatTemplateTest(openstack.OpenstackTest):

    template_dirs = [os.path.dirname(__file__)]
    template_file = 'my-stack.yaml'

    def test_get_template(self, template_file=None, template_dirs=None):
        template_file = template_file or self.template_file
        template_dirs = template_dirs or self.template_dirs
        template = heat.get_heat_template(template_file=template_file,
                                          template_dirs=template_dirs)
        self.assertIsInstance(template, heat.HeatTemplate)
        self.assertIsInstance(template.template, dict)
        self.assertEqual(
            os.path.join(os.path.dirname(__file__), template_file),
            template.file)



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import time

import netaddr
from neutron_lib import constants
from oslo_log import log

from tobiko.shell import sh
from tobiko.shell.ping import _exception
from tobiko.shell.ping import _parameters
from tobiko.shell.ping import _statistics


LOG = log.getLogger(__name__)


TRANSMITTED = 'transmitted'
DELIVERED = 'delivered'
UNDELIVERED = 'undelivered'
RECEIVED = 'received'
UNRECEIVED = 'unreceived'


def ping(host, until=TRANSMITTED, check=True, **ping_params):
    """Send ICMP messages to host address until timeout

    :param host: destination host address
    :param **ping_params: parameters to be forwarded to get_statistics()
        function
    :returns: PingStatistics
    """
    return get_statistics(host=host, until=until, check=check, **ping_params)


def ping_until_delivered(host, **ping_params):
    """Send 'count' ICMP messages

    Send 'count' ICMP messages

    ICMP messages are considered delivered when they have been
    transmitted without being counted as errors.

    :param host: destination host address
    :param **ping_params: parameters to be forwarded to get_statistics()
        function
    :returns: PingStatistics
    :raises: PingFailed in case timeout expires before delivering all
        expected count messages
    """
    return ping(host=host, until=DELIVERED, **ping_params)


def ping_until_undelivered(host, **ping_params):
    """Send ICMP messages until it fails to deliver messages

    Send ICMP messages until it fails to deliver 'count' messages

    ICMP messages are considered undelivered when they have been
    transmitted and they have been counted as error in ping statistics (for
    example because of errors into the route to remote address).

    :param host: destination host address
    :param **ping_params: parameters to be forwarded to get_statistics()
        function
    :returns: PingStatistics
    :raises: PingFailed in case timeout expires before failing delivering
        expected 'count' of messages
    """
    return ping(host=host, until=UNDELIVERED, **ping_params)


def ping_until_received(host, **ping_params):
    """Send ICMP messages until it receives messages back

    Send ICMP messages until it receives 'count' messages back

    ICMP messages are considered received when they have been
    transmitted without any routing errors and they are received back

    :param host: destination host address
    :param **ping_params: parameters to be forwarded to get_statistics()
        function
    :returns: PingStatistics
    :raises: PingFailed in case timeout expires before receiving all
        expected 'count' of messages
    """
    return ping(host=host, until=RECEIVED, **ping_params)


def ping_until_unreceived(host, **ping_params):
    """Send ICMP messages until it fails to receive messages

    Send ICMP messages until it fails to receive 'count' messages back.

    ICMP messages are considered unreceived when they have been
    transmitted without any routing error but they failed to be received
    back (for example because of network filtering).

    :param host: destination host address
    :param **ping_params: parameters to be forwarded to get_statistics()
        function
    :returns: PingStatistics
    :raises: PingFailed in case timeout expires before failed receiving
        expected 'count' of messages
    """
    return ping(host=host, until=UNRECEIVED, **ping_params)


def get_statistics(parameters=None, ssh_client=None, until=None, check=True,
                   **ping_params):
    parameters = _parameters.get_ping_parameters(default=parameters,
                                                 **ping_params)
    statistics = _statistics.PingStatistics()
    for partial_statistics in iter_statistics(parameters=parameters,
                                              ssh_client=ssh_client,
                                              until=until, check=check):
        statistics += partial_statistics
        LOG.debug("%r", statistics)

    return statistics


def iter_statistics(parameters=None, ssh_client=None, until=None, check=True,
                    **ping_params):
    parameters = _parameters.get_ping_parameters(default=parameters,
                                                 **ping_params)
    now = time.time()
    end_of_time = now + parameters.timeout
    deadline = parameters.deadline
    transmitted = 0
    received = 0
    undelivered = 0
    count = 0

    while deadline > 0 and count < parameters.count:
        # splitting total timeout interval into smaller deadline intervals will
        # cause ping command to be executed more times allowing to handle
        # temporary packets routing problems
        if until == RECEIVED:
            execute_parameters = _parameters.get_ping_parameters(
                default=parameters,
                deadline=deadline,
                count=(parameters.count - count))
        else:
            # Deadline ping parameter cause ping to be executed until count
            # messages are received or deadline is expired
            # Therefore to count messages not of received type we have to
            # simulate deadline parameter limiting the maximum number of
            # transmitted messages
            execute_parameters = _parameters.get_ping_parameters(
                default=parameters,
                deadline=deadline,
                count=min(parameters.count - count,
                          parameters.interval * deadline))

        # Command timeout would typically give ping command additional seconds
        # to safely reach deadline before shell command timeout expires, while
        # in the same time adding an extra verification to forbid using more
        # time than expected considering the time required to make SSH
        # connection and running a remote shell
        try:
            result = execute_ping(parameters=execute_parameters,
                                  ssh_client=ssh_client,
                                  timeout=end_of_time - now,
                                  check=check)
        except sh.ShellError as ex:
            LOG.exception("Error executing ping command")
            output = str(ex.stdout)

        else:
            output = str(result.stdout)

        if output:
            statistics = _statistics.parse_ping_statistics(
                output=output, begin_interval=now,
                end_interval=time.time())

            yield statistics

            transmitted += statistics.transmitted
            received += statistics.received
            undelivered += statistics.undelivered
            count = {None: 0,
                     TRANSMITTED: transmitted,
                     DELIVERED: transmitted - undelivered,
                     UNDELIVERED: undelivered,
                     RECEIVED: received,
                     UNRECEIVED: transmitted - received}[until]

        now = time.time()
        deadline = min(int(end_of_time - now), parameters.deadline)

    if until and count < parameters.count:
        raise _exception.PingFailed(count=count,
                                    expected_count=parameters.count,
                                    timeout=parameters.timeout,
                                    message_type=until)


def execute_ping(parameters, ssh_client=None, check=True, **params):
    if not ssh_client:
        is_cirros_image = params.setdefault('is_cirros_image', False)
        if is_cirros_image:
            raise ValueError("'ssh_client' parameter is required when "
                             "to execute ping on a CirrOS image.")

    command = get_ping_command(parameters)
    result = sh.execute(command=command, ssh_client=ssh_client,
                        timeout=parameters.timeout, check=False, wait=True)

    if check and result.exit_status and result.stderr:
        handle_ping_command_error(error=str(result.stderr))
    return result


def get_ping_command(parameters):
    options = []

    ip_version = parameters.ip_version
    host = parameters.host
    if host:
        try:
            host = netaddr.IPAddress(host)
        except netaddr.core.AddrFormatError:
            # NOTE: host could be an host name not an IP address so
            # this is fine
            LOG.debug("Unable to obtain IP version from host address: %r",
                      host)
        else:
            if ip_version != host.version:
                if ip_version:
                    raise ValueError("Mismatching destination IP version.")
                else:
                    ip_version = host.version
    else:
        raise ValueError("Ping host destination hasn't been specified")

    source = parameters.source
    if source:
        try:
            source = netaddr.IPAddress(source)
        except netaddr.core.AddrFormatError:
            # NOTE: source could be a device name and not an IP address
            # so this is fine
            LOG.debug("Unable to obtain IP version from source address: "
                      "%r", source)
        else:
            if ip_version != source.version:
                if ip_version:
                    raise ValueError("Mismatching source IP version.")
                else:
                    ip_version = source.version
        options += ['-I', source]

    is_cirros_image = parameters.is_cirros_image
    ping_command = 'ping'
    if not ip_version:
        LOG.warning("Unable to obtain IP version from neither source "
                    "or destination IP addresses: assuming IPv4")
        ip_version = constants.IP_VERSION_4

    elif ip_version == constants.IP_VERSION_6:
        if is_cirros_image:
            options += ['-6']
        else:
            ping_command = 'ping6'

    deadline = parameters.deadline
    if deadline > 0:
        options += ['-w', deadline]
        options += ['-W', deadline]

    count = parameters.count
    if count > 0:
        options += ['-c', int(count)]

    payload_size = parameters.payload_size
    packet_size = parameters.packet_size
    if not payload_size and packet_size:
        payload_size = get_icmp_payload_size(package_size=int(packet_size),
                                             ip_version=ip_version)

    if payload_size:
        options += ['-s', int(payload_size)]

    interval = parameters.interval
    if interval > 1:
        options += ['-i', int(interval)]

    fragmentation = parameters.fragmentation
    if fragmentation is False:
        if is_cirros_image:
            msg = ("'is_cirros_image' parameter must be set to False"
                   " when 'fragmention' parameter is False "
                   "(is_cirros_image={!r})").format(is_cirros_image)
            raise ValueError(msg)
        options += ['-M', 'do']

    return [ping_command] + options + [host]


def handle_ping_command_error(error):
    for error in error.splitlines():
        error = error.strip()
        if error:
            prefix = 'ping: '
            if error.startswith('ping: '):
                text = error[len(prefix):]

                prefix = 'bad address '
                if text.startswith(prefix):
                    address = text[len(prefix):].replace("'", '').strip()
                    raise _exception.BadAddressPingError(address=address)

                prefix = 'local error: '
                if text.startswith(prefix):
                    details = text[len(prefix):].strip()
                    raise _exception.LocalPingError(details=details)

                prefix = 'sendto: '
                if text.startswith(prefix):
                    details = text[len(prefix):].strip()
                    raise _exception.SendToPingError(details=details)

                prefix = 'unknown host '
                if text.startswith(prefix):
                    details = text[len(prefix):].strip()
                    raise _exception.UnknowHostError(details=details)

                suffix = ': Name or service not known'
                if text.endswith(suffix):
                    details = text[:-len(suffix)].strip()
                    raise _exception.UnknowHostError(details=details)

                suffix = ': No route to host'
                if text.endswith(suffix):
                    details = text[:-len(suffix)].strip()
                    raise _exception.UnknowHostError(details=details)

                raise _exception.PingError(details=text)


IP_HEADER_SIZE = {4: 20, 6: 40}
ICMP_HEADER_SIZE = {4: 8, 6: 4}


def get_icmp_payload_size(package_size, ip_version):
    """Return the maximum size of ping payload that will fit into MTU."""
    header_size = IP_HEADER_SIZE[ip_version] + ICMP_HEADER_SIZE[ip_version]
    if package_size < header_size:
        message = ("package size {package_size!s} is smaller than package "
                   "header size {header_size!s}").format(
                       package_size=package_size,
                       header_size=header_size)
        raise ValueError(message)
    return package_size - header_size



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from oslo_config import cfg


def register_tobiko_options(conf):

    conf.register_opts(
        group=cfg.OptGroup('keystone'),
        opts=[cfg.IntOpt('api_version',
                         default=None,
                         help="Identity API version"),
              cfg.StrOpt('auth_url',
                         default=None,
                         help="Identity service URL"),
              cfg.StrOpt('username',
                         default=None,
                         help="Username"),
              cfg.StrOpt('project_name',
                         default=None,
                         help="Project name"),
              cfg.StrOpt('password',
                         default=None,
                         help="Password"),
              cfg.StrOpt('domain_name',
                         default=None,
                         help="Domain name"),
              cfg.StrOpt('user_domain_name',
                         default=None,
                         help="User domain name"),
              cfg.StrOpt('project_domain_name',
                         default=None,
                         help="Project domain name"),
              cfg.StrOpt('project_domain_id',
                         default=None,
                         help="Project domain ID"),
              cfg.StrOpt('trust_id',
                         default=None,
                         help="Trust ID for trust scoping.")])



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from keystoneauth1 import loading
from keystoneauth1 import session as _session
from oslo_log import log

import tobiko
from tobiko.openstack.keystone import _credentials


LOG = log.getLogger(__name__)


class KeystoneSessionFixture(tobiko.SharedFixture):

    session = None
    credentials = None
    credentials_fixture = None

    def __init__(self, credentials=None):
        super(KeystoneSessionFixture, self).__init__()
        if credentials:
            if tobiko.is_fixture(credentials):
                self.credentials_fixture = credentials
            else:
                self.credentials = credentials

    def setup_fixture(self):
        self.setup_credentials()
        self.setup_session(credentials=self.credentials)

    def setup_credentials(self):
        credentials_fixture = self.credentials_fixture
        if credentials_fixture:
            self.credentials = tobiko.setup_fixture(
                credentials_fixture).credentials
        elif not self.credentials:
            self.credentials = _credentials.default_keystone_credentials()

    def setup_session(self, credentials):
        LOG.debug("Create session for credentials %r", credentials)
        loader = loading.get_plugin_loader('password')
        params = credentials.to_dict()
        del params['api_version']  # parameter not required
        auth = loader.load_from_options(**params)
        self.session = _session.Session(auth=auth, verify=False)


class KeystoneSessionManager(object):

    def __init__(self):
        self._sessions = {}

    def get_session(self, credentials=None, shared=True, init_session=None):
        if shared:
            key = credentials
            if credentials:
                if tobiko.is_fixture(credentials):
                    key = tobiko.get_fixture_name(credentials)

            session = self._sessions.get(key)
            if session:
                return session

        init_session = init_session or KeystoneSessionFixture
        assert callable(init_session)
        LOG.debug('Initialize Keystone session: %r(credentials=%r)',
                  init_session, credentials)
        session = init_session(credentials=credentials)

        if shared:
            self._sessions[key] = session
        return session


SESSIONS = KeystoneSessionManager()


def get_keystone_session(credentials=None, shared=True, init_session=None,
                         manager=None):
    manager = manager or SESSIONS
    session = manager.get_session(credentials=credentials, shared=shared,
                                  init_session=init_session)
    session.setUp()
    return session.session



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import argparse
import logging
import sys

from tobiko.cmd import base

LOG = logging.getLogger(__name__)


class ListUtil(base.TobikoCMD):

    def __init__(self):
        super(ListUtil, self).__init__()
        self.parser = self.get_parser()
        self.args = (self.parser).parse_args()

    def get_parser(self):
        parser = argparse.ArgumentParser(add_help=True)
        parser.add_argument('--playbooks', '-p',
                            help="List playbooks provided by Tobiko",
                            const='list_playbooks',
                            action='store_const', dest='action')
        return parser

    def list_playbooks(self):
        """Lists playbooks included in Tobiko."""
        for playbook in self.ansibleManager.get_playbooks_names():
            sys.stdout.write(playbook + '\n')


def main():
    """List CLI main entry."""
    list_cmd = ListUtil()
    if list_cmd.args.action:
        action_func = getattr(list_cmd, list_cmd.args.action)
        action_func()
    else:
        list_cmd.list_playbooks()


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os
import sys

from oslo_log import log
from stestr import config_file

LOG = log.getLogger(__name__)

os.environ.setdefault('PYTHON', sys.executable)


def discover_testcases(manager=None, **kwargs):
    manager = manager or TESTCASES
    return manager.discover(**kwargs)


class TestCaseManager(object):

    def __init__(self, config=None, repo_type=None, repo_url=None,
                 test_path=None, top_dir=None, group_regex=None,
                 blacklist_file=None, whitelist_file=None, black_regex=None,
                 filters=None):
        """
        :param str config: The path to the stestr config file. Must be a
            string.
        :param str repo_type: This is the type of repository to use. Valid
            choices are 'file' and 'sql'.
        :param str repo_url: The url of the repository to use.
        :param str test_path: Set the test path to use for unittest discovery.
            If both this and the corresponding config file option are set, this
            value will be used.
        :param str top_dir: The top dir to use for unittest discovery. This
            takes precedence over the value in the config file. (if one is
            present in the config file)
        :param str group_regex: Set a group regex to use for grouping tests
            together in the stestr scheduler. If both this and the
            corresponding config file option are set this value will be used.
        :param str blacklist_file: Path to a blacklist file, this file contains
            a separate regex exclude on each newline.
        :param str whitelist_file: Path to a whitelist file, this file contains
            a separate regex on each newline.
        :param str black_regex: Test rejection regex. If a test cases name
            matches on re.search() operation, it will be removed from the final
            test list.
        :param list filters: A list of string regex filters to initially apply
            on the test list. Tests that match any of the regexes will be used.
            (assuming any other filtering specified also uses it)
        """

        self.config = config or '.stestr.conf'
        self.repo_type = repo_type or 'file'
        self.repo_url = repo_url
        self.test_path = test_path
        self.top_dir = top_dir
        self.group_regex = group_regex
        self.blacklist_file = blacklist_file
        self.whitelist_file = whitelist_file
        self.black_regex = black_regex
        self.filters = filters

    def discover(self, **kwargs):
        """Iterate over test_ids for a project
        This method will print the test_ids for tests in a project. You can
        filter the output just like with the run command to see exactly what
        will be run.
        """
        params = dict(config=self.config, repo_type=self.repo_type,
                      repo_url=self.repo_url, test_path=self.test_path,
                      top_dir=self.top_dir, group_regex=self.group_regex,
                      blacklist_file=self.blacklist_file,
                      whitelist_file=self.whitelist_file,
                      black_regex=self.black_regex, filters=self.filters)
        if kwargs:
            params.update(kwargs)
        ids = None
        config = params.pop('config')
        conf = config_file.TestrConf(config)
        filters = params.pop('filters')
        blacklist_file = params.pop('blacklist_file')
        whitelist_file = params.pop('whitelist_file')
        black_regex = params.pop('black_regex')
        cmd = conf.get_run_command(
            regexes=filters, repo_type=params['repo_type'],
            repo_url=params['repo_url'], group_regex=params['group_regex'],
            blacklist_file=blacklist_file, whitelist_file=whitelist_file,
            black_regex=black_regex, test_path=params['test_path'],
            top_dir=params['top_dir'])
        not_filtered = filters is None and blacklist_file is None\
            and whitelist_file is None and black_regex is None

        try:
            cmd.setUp()
            # List tests if the fixture has not already needed to to filter.
            if not_filtered:
                ids = cmd.list_tests()
            else:
                ids = cmd.test_ids
        except SystemExit:
            msg = ("Error discovering test cases IDs with parameters: "
                   "{!r}").format(params)
            raise RuntimeError(msg)
        finally:
            cmd.cleanUp()

        return sorted(ids)


TESTCASES = TestCaseManager()



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import shutil
import tempfile

import mock
from oslo_log import log

from tobiko.common import _fixture
from tobiko.tests import base


class TobikoUnitTest(base.TobikoTest):

    def setUp(self):
        super(TobikoUnitTest, self).setUp()
        # Protect from mis-configuring logging
        self.patch(log, 'setup')
        self.fixture_manager = manager = _fixture.FixtureManager()
        self.patch(_fixture, 'FIXTURES', manager)

    def patch(self, obj, attribute, value=mock.DEFAULT, spec=None,
              create=False, spec_set=None, autospec=None,
              new_callable=None, **kwargs):
        # pylint: disable=arguments-differ
        context = mock.patch.object(target=obj, attribute=attribute, new=value,
                                    spec=spec, create=create,
                                    spec_set=spec_set, autospec=autospec,
                                    new_callable=new_callable, **kwargs)
        mocked = context.start()
        self.addCleanup(context.stop)
        return mocked

    def create_tempdir(self, *args, **kwargs):
        dir_path = tempfile.mkdtemp(*args, **kwargs)
        self.addCleanup(shutil.rmtree(dir_path, ignore_errors=True))
        return dir_path



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from heatclient import client as heatclient

from tobiko.openstack import _client


class HeatClientFixture(_client.OpenstackClientFixture):

    def init_client(self, session):
        return heatclient.Client(
            '1', session=session, endpoint_type='public',
            service_type='orchestration')


CLIENTS = _client.OpenstackClientManager(init_client=HeatClientFixture)


def get_heat_client(session=None, shared=True, init_client=None,
                    manager=None):
    manager = manager or CLIENTS
    client = manager.get_client(session=session, shared=shared,
                                init_client=init_client)
    client.setUp()
    return client.client



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from tobiko.common import _asserts
from tobiko.common import _exception
from tobiko.common import _fixture
from tobiko.common.managers import testcase as testcase_manager
from tobiko.common.managers import loader as loader_manager
from tobiko.common import _skip


FailureException = _asserts.FailureException
fail = _asserts.fail

TobikoException = _exception.TobikoException

is_fixture = _fixture.is_fixture
get_fixture = _fixture.get_fixture
fixture_property = _fixture.fixture_property
required_fixture = _fixture.required_fixture
required_setup_fixture = _fixture.required_setup_fixture
get_fixture_name = _fixture.get_fixture_name
get_fixture_class = _fixture.get_fixture_class
get_fixture_dir = _fixture.get_fixture_dir
remove_fixture = _fixture.remove_fixture
setup_fixture = _fixture.setup_fixture
cleanup_fixture = _fixture.cleanup_fixture
list_required_fixtures = _fixture.list_required_fixtures
SharedFixture = _fixture.SharedFixture

load_object = loader_manager.load_object
load_module = loader_manager.load_module

discover_testcases = testcase_manager.discover_testcases

SkipException = _skip.SkipException
skip = _skip.skip
skip_if = _skip.skip_if
skip_until = _skip.skip_until



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import six

from tobiko.shell.ssh import _config


def ssh_login(hostname, username=None, port=None):
    login = hostname
    if port:
        login += ':' + str(port)
    if username:
        login = username + '@' + login
    return login


def ssh_command(host, username=None, port=None, command=None,
                config_files=None, host_config=None, **options):
    host_config = host_config or _config.ssh_host_config(
        host=host, config_files=config_files)

    command = command or host_config.default.command.split()
    if isinstance(command, six.string_types):
        command = command.split()

    hostname = host_config.hostname
    username = username or host_config.username
    command += [ssh_login(hostname=hostname, username=username)]

    #     if host_config.default.debug:
    #         command += ['-vvvvvv']

    port = port or host_config.port
    if port:
        command += ['-p', port]

    for name, value in host_config.host_config.items():
        if name not in {'hostname', 'port', 'user'}:
            options.setdefault(name, value)
    options.setdefault('userknownhostsfile', '/dev/null')
    options.setdefault('stricthostkeychecking', 'no')
    options.setdefault('loglevel', 'quiet')
    options.setdefault('connecttimeout', int(host_config.timeout))
    options.setdefault('connectionattempts', host_config.connection_attempts)
    if options:
        for name, value in sorted(options.items()):
            command += ['-o', '{!s}={!s}'.format(name, value)]

    return command



# Copyright (c) 2018 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import argparse
import sys

from tobiko.cmd import fault as _fault
from tobiko.tests import unit


class FaultCMDTest(unit.TobikoUnitTest):

    command_name = 'tobiko-fault'
    command_class = _fault.FaultCMD
    default_fault = ["some_fault"]

    def setUp(self):
        super(FaultCMDTest, self).setUp()
        self.mock_error = self.patch(argparse.ArgumentParser, 'error',
                                     side_effect=self.fail)

    def patch_argv(self, arguments=None):
        """Patch argv"""
        arguments = list(arguments or [])
        if not arguments:
            arguments = self.default_fault
        return self.patch(sys, 'argv',
                          [self.command_name] + arguments)

    def test_init(self, arguments=None):
        self.patch_argv(arguments=arguments)
        command = self.command_class()
        self.mock_error.assert_not_called()
        args = command.args
        self.assertIsNotNone(args)



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

import six

from tobiko import config
from tobiko.openstack import heat
from tobiko.openstack.stacks import _hot


CONF = config.CONF


class NovaKeyPairStackFixture(heat.HeatStackFixture):
    template = _hot.heat_template_file('nova/key_pair.yaml')
    key_file = os.path.expanduser(CONF.tobiko.nova.key_file)
    public_key = None
    private_key = None

    def setup_fixture(self):
        self.read_keys()
        super(NovaKeyPairStackFixture, self).setup_fixture()

    def read_keys(self):
        with open(self.key_file, 'r') as fd:
            self.private_key = as_str(fd.read())
        with open(self.key_file + '.pub', 'r') as fd:
            self.public_key = as_str(fd.read())


def as_str(text):
    if isinstance(text, six.string_types):
        return text
    else:
        return text.decode()



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import


import tobiko
from tobiko import config
from tobiko.openstack import heat
from tobiko.openstack.stacks import _hot
from tobiko.openstack.stacks import _nova
from tobiko.shell import ssh


CONF = config.CONF


class NeutronNetworkStackFixture(heat.HeatStackFixture):
    """Heat stack for creating internal network with a router to external

    """

    #: Heat template file
    template = _hot.heat_template_file('neutron/network.yaml')

    #: IPv4 sub-net CIDR
    ipv4_cidr = '190.40.2.0/24'

    @property
    def has_ipv4(self):
        return bool(self.ipv4_cidr)

    #: IPv6 sub-net CIDR
    ipv6_cidr = '2001:db8:1:2::/64'

    @property
    def has_ipv6(self):
        return bool(self.ipv6_cidr)

    #: Floating IP network where the Neutron floating IPs are created
    gateway_network = CONF.tobiko.neutron.floating_network

    @property
    def has_gateway(self):
        return bool(self.gateway_network)


class NeutronServerStackFixture(heat.HeatStackFixture):

    #: Heat template file
    template = _hot.heat_template_file('neutron/server.yaml')

    key_pair_stack = tobiko.required_setup_fixture(
        _nova.NovaKeyPairStackFixture)
    network_stack = tobiko.required_setup_fixture(NeutronNetworkStackFixture)

    #: Glance image used to create a Nova server instance
    image = CONF.tobiko.nova.image

    #: Nova flavor used to create a Nova server instance
    flavor = CONF.tobiko.nova.flavor

    #: username used to login to a Nova server instance
    username = CONF.tobiko.nova.username

    #: password used to login to a Nova server instance
    password = CONF.tobiko.nova.password

    @property
    def key_name(self):
        return self.key_pair_stack.outputs.key_name

    @property
    def network(self):
        return self.network_stack.outputs.network_id

    #: Floating IP network where the Neutron floating IP is created
    floating_network = CONF.tobiko.neutron.floating_network

    @property
    def has_floating_ip(self):
        return bool(self.floating_network)

    @property
    def ssh_client(self):
        return ssh.ssh_client(
            host=self.outputs.floating_ip_address,
            username=self.username,
            password=self.password)

    @property
    def ssh_command(self):
        return ssh.ssh_command(
            host=self.outputs.floating_ip_address,
            username=self.username)



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import getpass

from oslo_config import cfg
from oslo_log import log


def register_tobiko_options(conf):
    conf.register_opts(
        group=cfg.OptGroup('ssh'),
        opts=[cfg.BoolOpt('debug',
                          default=False,
                          help=('Logout debugging messages of paramiko '
                                'library')),
              cfg.StrOpt('command',
                         default='/usr/bin/ssh',
                         help=('Default SSH client command')),
              cfg.StrOpt('port',
                         default=22,
                         help=('Default SSH port')),
              cfg.StrOpt('username',
                         default=getpass.getuser(),
                         help=('Default SSH username')),
              cfg.ListOpt('config_files',
                          default=['/etc/ssh/ssh_config', '~/.ssh/config'],
                          help="Default user SSH configuration files"),
              cfg.StrOpt('key_file',
                         default='~/.ssh/id_rsa',
                         help="Default SSH private key file"),
              cfg.BoolOpt('allow_agent',
                          default=False,
                          help=("Set to False to disable connecting to the "
                                "SSH agent")),
              cfg.BoolOpt('compress',
                          default=False,
                          help="Set to True to turn on compression"),
              cfg.FloatOpt('timeout',
                           default=5.,
                           help="SSH connect timeout in seconds"),
              cfg.IntOpt('connection_attempts',
                         default=24,
                         help=("Incremental seconds to wait after every "
                               "failed SSH connection attempt")),
              cfg.FloatOpt('connection_interval',
                           default=5.,
                           help=("Minimal seconds to wait between every "
                                 "failed SSH connection attempt")),
              cfg.StrOpt('proxy_jump',
                         default=None,
                         help="Default SSH proxy server"),
              cfg.StrOpt('proxy_command',
                         default=None,
                         help="Default proxy command"),
              ])


def setup_tobiko_config(conf):
    paramiko_logger = log.getLogger('paramiko')
    if conf.ssh.debug:
        if not paramiko_logger.isEnabledFor(log.DEBUG):
            # Print paramiko debugging messages
            paramiko_logger.logger.setLevel(log.DEBUG)
    elif paramiko_logger.isEnabledFor(log.DEBUG):
        # Silence paramiko debugging messages
        paramiko_logger.logger.setLevel(log.WARNING)



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from tobiko.openstack.neutron import _client
from tobiko.openstack.neutron import _extension


get_neutron_client = _client.get_neutron_client
NeutronClientFixture = _client.NeutronClientFixture

get_networking_extensions = _extension.get_networking_extensions
missing_networking_extensions = _extension.missing_networking_extensions
has_networking_extensions = _extension.has_networking_extensions
skip_if_missing_networking_extensions = (
    _extension.skip_if_missing_networking_extensions)



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import netaddr
import testtools

from tobiko import config
from tobiko.shell import ping


CONF = config.CONF


class PingTest(testtools.TestCase):

    def test_ping_recheable_address(self):
        result = ping.ping('127.0.0.1', count=3)
        self.assertIsNone(result.source)
        self.assertEqual(netaddr.IPAddress('127.0.0.1'), result.destination)
        result.assert_transmitted()
        result.assert_replied()

    def test_ping_reachable_hostname(self):
        result = ping.ping('example.org', count=3)
        self.assertIsNone(result.source)
        # self.assertIsNotNone(result.destination)
        result.assert_transmitted()
        result.assert_replied()

    def test_ping_unreachable_address(self):
        result = ping.ping('1.2.3.4', count=3)
        self.assertIsNone(result.source)
        self.assertEqual(netaddr.IPAddress('1.2.3.4'), result.destination)
        result.assert_transmitted()
        result.assert_not_replied()

    def test_ping_unreachable_hostname(self):
        ex = self.assertRaises(ping.UnknowHostError, ping.ping,
                               'unreachable-host', count=3)
        self.assertEqual('unreachable-host', ex.details)

    def test_ping_until_received(self):
        result = ping.ping_until_received('127.0.0.1', count=3)
        self.assertIsNone(result.source)
        self.assertEqual(netaddr.IPAddress('127.0.0.1'), result.destination)
        result.assert_transmitted()
        result.assert_replied()

    def test_ping_until_received_unreachable(self):
        ex = self.assertRaises(ping.PingFailed, ping.ping_until_received,
                               '1.2.3.4', count=3, timeout=6)
        self.assertEqual(6, ex.timeout)
        self.assertEqual(0, ex.count)
        self.assertEqual(3, ex.expected_count)
        self.assertEqual('received', ex.message_type)

    def test_ping_until_unreceived_recheable(self):
        ex = self.assertRaises(ping.PingFailed, ping.ping_until_unreceived,
                               '127.0.0.1', count=3, timeout=6)
        self.assertEqual(6, ex.timeout)
        self.assertEqual(0, ex.count)
        self.assertEqual(3, ex.expected_count)
        self.assertEqual('unreceived', ex.message_type)

    def test_ping_until_unreceived_unrecheable(self):
        result = ping.ping_until_unreceived('1.2.3.4', count=3)
        self.assertIsNone(result.source)
        self.assertEqual(netaddr.IPAddress('1.2.3.4'), result.destination)
        result.assert_transmitted()
        result.assert_not_replied()



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import


class PingParameters(object):
    """Recollect parameters to be used to format ping command line

    PingParameters class is a data model recollecting parameters used to
    create a ping command line. It provides the feature of copying default
    values from another instance of PingParameters passed using constructor
    parameter 'default'.
    """

    def __init__(self, host=None, count=None, deadline=None,
                 fragmentation=None, interval=None, is_cirros_image=None,
                 ip_version=None, payload_size=None, packet_size=None,
                 source=None, timeout=None):
        self.count = count
        self.deadline = deadline
        self.host = host
        self.fragmentation = fragmentation
        self.interval = interval
        self.ip_version = ip_version
        self.is_cirros_image = is_cirros_image
        self.packet_size = packet_size
        self.payload_size = payload_size
        self.source = source
        self.timeout = timeout

    def __repr__(self):
        return "PingParameters({!s})".format(
            ", ".join("{!s}={!r}".format(k, v)
                      for k, v in self.__dict__.items()))


def get_ping_parameters(default=None, **ping_params):
    """Get ping parameters eventually merging them with given extra parameters

    The only difference with init_parameters function is that in case
    default parameter is not None not any extra parameters is given, then
    it simply return given default instance, without performing any validation.
    """
    if default and not ping_params:
        return default
    else:
        return ping_parameters(default=default, **ping_params)


def ping_parameters(default=None, count=None, deadline=None,
                    fragmentation=None, host=None, interval=None,
                    is_cirros_image=None, ip_version=None, packet_size=None,
                    payload_size=None, source=None, timeout=None):
    """Validate parameters and initialize a new PingParameters instance

    :param default: (PingParameters or None) instance from where to take
    default values when other value is not provided. If None (that is the
    default value) it will copy default parameters from
    DEFAULT_PING_PARAMETERS

    :param count: (int or None) number of ping ICMP message expecting to be
    received before having a success. Default value can be configured using
    'count' option in [ping] config section.

    :param host: (str or None) IP address or host name to send ICMP
    messages to. It is required to format a valid ping command, therefore
    no default value exists for this parameter.

    :param deadline: (int or None) positive number representing the maximum
    number of seconds ping command can send ICMP messages before stop
    executing. Default value can be configured using 'deadline' option
    in [ping] config section.

    :param fragmentation: (bool or None) when False this would tell ping
    to forbid ICMP messages fragmentation. Default value can be configured
    using 'fragmentation' option in [ping] config section.

    :param interval: (int or None) interval of time before sending following
    ICMP message. Default value can be configured using 'interval' option
    in [ping] config section.

    :param is_cirros_image: (bool or None) when True means that ping command
    has to be formated for being executed on a CirrOS based guess instance.

    :param ip_version: (4, 6 or None) If not None it makes sure it will
    use specified IP version for sending ICMP packages.

    :param packet_size: ICMP message size. Default value can be configured
    using 'package_size' option in [ping] config section.

    :param payload_size: (int or None) if not None, it specifies ICMP message
    size minus ICMP and IP header size.

    :param source: (str or None) IP address or interface name from where
    to send ICMP message.

    :param timeout: (int or None) time in seconds after which ping operation
    would raise PingFailed exception.

    :raises TypeError: in case some parameter cannot be converted to right
    expected type

    :raises ValueError: in case some parameter has an unexpected value
    """

    if packet_size:
        if payload_size:
            msg = ("Can't set 'package_size' and 'payload_size' parameters "
                   "at the same time: package_size={!r}, payload_size={!r}"
                   ).format(packet_size, payload_size)
            raise ValueError(msg)

    count = count or 1
    if count < 1:
        msg = ("Count is not positive: count={!r}").format(count)
        raise ValueError(msg)

    if default is not False:
        default = default or default_ping_parameters()
        # Copy default parameters
        count = count or default.count
        if deadline is None:
            deadline = default.deadline
        host = host or default.host
        if fragmentation is None:
            fragmentation = default.fragmentation
        interval = interval or default.interval
        ip_version = ip_version or default.ip_version
        packet_size = packet_size or default.packet_size
        payload_size = payload_size or default.payload_size
        source = source or default.source
        timeout = timeout or default.timeout

    count = int(count)
    if count < 1:
        msg = "'count' parameter cannot be smaller than 1"
        raise ValueError(msg)

    deadline = int(deadline)
    if deadline < 0:
        msg = ("'deadline' parameter cannot be smaller than 0 "
               "(deadline={!r})").format(deadline)
        raise ValueError(msg)

    interval = int(interval)
    if interval < 1:
        msg = ("'interval' parameter cannot be smaller than 1 "
               "(interval={!r})").format(interval)
        raise ValueError(msg)

    timeout = int(timeout)
    if timeout < 1:
        msg = ("'timeout' parameter cannot be smaller than 1 "
               "(timeout={!r})").format(timeout)
        raise ValueError(msg)

    if is_cirros_image:
        if fragmentation is False:
            msg = ("'fragmentation' parameter cannot be False when "
                   "pinging from a CirrOS image (is_cirros_image={!r})"
                   ).format(is_cirros_image)
            raise ValueError(msg)

        if interval != 1:
            msg = ("Cannot specify 'interval' parameter when pinging from a "
                   "CirrOS image (interval={!r}, is_cirros_image={!r})"
                   ).format(interval, is_cirros_image)
            raise ValueError(msg)

    return PingParameters(count=count, host=host,
                          deadline=deadline, fragmentation=fragmentation,
                          interval=interval, is_cirros_image=is_cirros_image,
                          ip_version=ip_version, packet_size=packet_size,
                          payload_size=payload_size, source=source,
                          timeout=timeout)


def default_ping_parameters():
    from tobiko import config
    CONF = config.CONF
    return ping_parameters(
        default=False,
        count=CONF.tobiko.ping.count,
        deadline=CONF.tobiko.ping.deadline,
        fragmentation=CONF.tobiko.ping.fragmentation,
        interval=CONF.tobiko.ping.interval,
        packet_size=CONF.tobiko.ping.packet_size,
        timeout=CONF.tobiko.ping.timeout)



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import fcntl
import subprocess
import os

from oslo_log import log
import paramiko
import six

import tobiko
from tobiko.shell import ssh
from tobiko.shell.sh import _command
from tobiko.shell.sh import _process


LOG = log.getLogger(__name__)


DATA_TYPES = six.string_types + (six.binary_type, six.text_type)


def execute(command, environment=None, timeout=None, shell=None, check=True,
            wait=None, stdin=True, stdout=True, stderr=True, ssh_client=None,
            **kwargs):
    """Execute command inside a remote or local shell

    :param command: command argument list

    :param timeout: command execution timeout in seconds

    :param check: when False it doesn't raises ShellCommandError when
    exit status is not zero. True by default

    :param ssh_client: SSH client instance used for remote shell execution

    :raises ShellTimeoutExpired: when timeout expires before command execution
    terminates. In such case it kills the process, then it eventually would
    try to read STDOUT and STDERR buffers (not fully implemented) before
    raising the exception.

    :raises ShellCommandError: when command execution terminates with non-zero
    exit status.
    """

    fixture = ShellExecuteFixture(
        command, environment=environment, shell=shell, stdin=stdin,
        stdout=stdout, stderr=stderr, timeout=timeout, check=check, wait=wait,
        ssh_client=ssh_client, **kwargs)
    return tobiko.setup_fixture(fixture).process


def local_execute(command, environment=None, shell=None, stdin=True,
                  stdout=True, stderr=True, timeout=None, check=True,
                  wait=None, **kwargs):
    """Execute command on local host using local shell"""

    return execute(
        command=command, environment=environment, shell=shell, stdin=stdin,
        stdout=stdout, stderr=stderr, timeout=timeout, check=check, wait=wait,
        ssh_client=False, **kwargs)


def ssh_execute(ssh_client, command, environment=None, shell=None, stdin=True,
                stdout=True, stderr=True, timeout=None, check=True, wait=None,
                **kwargs):
    """Execute command on local host using local shell"""
    return execute(
        command=command, environment=environment, shell=shell, stdin=stdin,
        stdout=stdout, stderr=stderr, timeout=timeout, check=check, wait=wait,
        ssh_client=ssh_client, **kwargs)


class ShellExecuteFixture(tobiko.SharedFixture):

    command = None
    shell = None
    environment = {}
    stdin = None
    stderr = None
    stdout = None
    timeout = 120.
    check = None
    wait = None
    process = None
    process_parameters = None

    def __init__(self, command=None, shell=None, environment=None, stdin=None,
                 stdout=None, stderr=None, timeout=None, check=None, wait=None,
                 ssh_client=None, **kwargs):
        super(ShellExecuteFixture, self).__init__()

        if ssh_client is not None:
            self.ssh_client = ssh_client
        else:
            self.ssh_client = ssh_client = self.default_ssh_client

        if shell is not None:
            self.shell = shell = bool(shell) and _command.shell_command(shell)
        elif not ssh_client:
            self.shell = shell = self.default_shell_command

        if command is None:
            command = self.command
        command = _command.shell_command(command)
        if shell:
            command = shell + [str(command)]
        self.command = command

        environment = environment or self.environment
        if environment:
            self.environment = dict(environment).update(environment)

        if stdin is not None:
            self.stdin = stdin
        if stdout is not None:
            self.stdout = stdout
        if stderr is not None:
            self.stderr = stderr
        if timeout is not None:
            self.timeout = timeout
        if check is not None:
            self.check = check
        if wait is not None:
            self.wait = wait

        self.process_parameters = (self.process_parameters and
                                   dict(self.process_parameters) or
                                   {})
        if kwargs:
            self.process_parameters.update(kwargs)

    @property
    def default_shell_command(self):
        from tobiko import config
        CONF = config.CONF
        return _command.shell_command(CONF.tobiko.shell.command)

    @property
    def default_ssh_client(self):
        return ssh.ssh_proxy_client()

    def setup_fixture(self):
        self.setup_process()

    def setup_process(self):
        self.process = self.execute()

    def execute(self, timeout=None, stdin=None, stdout=None, stderr=None,
                check=None, ssh_client=None, wait=None, **kwargs):
        command = self.command
        environment = self.environment
        if timeout is None:
            timeout = self.timeout
        LOG.debug("Execute command '%s' on local host (timeout=%r, "
                  "environment=%r)...",
                  command, timeout, environment)

        if stdin is None:
            stdin = self.stdin
        if stdout is None:
            stdout = self.stdout
        if stderr is None:
            stderr = self.stderr
        if check is None:
            check = self.check
        if wait is None:
            wait = self.wait
        if ssh_client is None:
            ssh_client = self.ssh_client

        process_parameters = self.process_parameters
        if kwargs:
            process_parameters = dict(process_parameters, **kwargs)

        process = self.create_process(command=command,
                                      environment=environment,
                                      timeout=timeout, stdin=stdin,
                                      stdout=stdout, stderr=stderr,
                                      ssh_client=ssh_client,
                                      **process_parameters)
        self.addCleanup(process.close)

        if stdin and isinstance(stdin, DATA_TYPES):
            process.send(data=stdin)

        if wait or check:
            if process.stdin:
                process.stdin.close()
            process.wait()
            if check:
                process.check_exit_status()

        return process

    def create_process(self, ssh_client, **kwargs):
        if ssh_client:
            return self.create_ssh_process(ssh_client=ssh_client, **kwargs)
        else:
            return self.create_local_process(**kwargs)

    def create_local_process(self, command, environment, timeout, stdin,
                             stdout, stderr, **kwargs):
        popen_params = {}
        if stdin:
            popen_params.update(stdin=subprocess.PIPE)
        if stdout:
            popen_params.update(stdout=subprocess.PIPE)
        if stderr:
            popen_params.update(stderr=subprocess.PIPE)
        process = subprocess.Popen(command,
                                   universal_newlines=True,
                                   env=environment,
                                   **popen_params)
        if stdin:
            set_non_blocking(process.stdin.fileno())
            kwargs.update(stdin=process.stdin)
        if stdout:
            set_non_blocking(process.stdout.fileno())
            kwargs.update(stdout=process.stdout)
        if stderr:
            set_non_blocking(process.stderr.fileno())
            kwargs.update(stderr=process.stderr)
        return LocalShellProcess(process=process, command=command,
                                 timeout=timeout, **kwargs)

    def create_ssh_process(self, command, environment, timeout, stdin, stdout,
                           stderr, ssh_client, **kwargs):
        """Execute command on a remote host using SSH client"""
        if isinstance(ssh_client, ssh.SSHClientFixture):
            # Connect to SSH server
            ssh_client = ssh_client.connect()
        if not isinstance(ssh_client, paramiko.SSHClient):
            message = "Object {!r} is not an SSHClient".format(ssh_client)
            raise TypeError(message)

        LOG.debug("Execute command %r on remote host (timeout=%r)...",
                  str(command), timeout)
        channel = ssh_client.get_transport().open_session()
        if environment:
            channel.update_environment(environment)
        channel.exec_command(str(command))
        if stdin:
            kwargs.update(stdin=StdinSSHChannelFile(channel, 'wb'))
        if stdout:
            kwargs.update(stdout=StdoutSSHChannelFile(channel, 'rb'))
        if stderr:
            kwargs.update(stderr=StderrSSHChannelFile(channel, 'rb'))
        return SSHShellProcess(channel=channel, command=command,
                               timeout=timeout, **kwargs)


def set_non_blocking(fd):
    flag = fcntl.fcntl(fd, fcntl.F_GETFL)
    fcntl.fcntl(fd, fcntl.F_SETFL, flag | os.O_NONBLOCK)


class LocalShellProcess(_process.ShellProcess):

    def __init__(self, process=None, **kwargs):
        super(LocalShellProcess, self).__init__(**kwargs)
        self.process = process

    def poll_exit_status(self):
        return self.process.poll()

    def kill(self):
        self.process.kill()


class SSHChannelFile(paramiko.ChannelFile):

    def fileno(self):
        return self.channel.fileno()


class StdinSSHChannelFile(SSHChannelFile):

    def close(self):
        super(StdinSSHChannelFile, self).close()
        self.channel.shutdown_write()

    @property
    def write_ready(self):
        return self.channel.send_ready()

    def write(self, data):
        super(StdinSSHChannelFile, self).write(data)
        return len(data)


class StdoutSSHChannelFile(SSHChannelFile):

    def fileno(self):
        return self.channel.fileno()

    def close(self):
        super(StdoutSSHChannelFile, self).close()
        self.channel.shutdown_read()

    @property
    def read_ready(self):
        return self.channel.recv_ready()


class StderrSSHChannelFile(SSHChannelFile, paramiko.channel.ChannelStderrFile):

    def fileno(self):
        return self.channel.fileno()

    @property
    def read_ready(self):
        return self.channel.recv_stderr_ready()


class SSHShellProcess(_process.ShellProcess):

    def __init__(self, channel=None, **kwargs):
        super(SSHShellProcess, self).__init__(**kwargs)
        self.channel = channel

    def poll_exit_status(self):
        if self.channel.exit_status_ready():
            return self.channel.recv_exit_status()

    def close(self):
        super(SSHShellProcess, self).close()
        self.channel.close()



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import sys

from oslo_log import log

import tobiko
from tobiko.cmd import base

LOG = log.getLogger(__name__)


class CreateUtil(base.TobikoCMD):

    def get_parser(self):
        parser = super(CreateUtil, self).get_parser()
        parser.add_argument(
            '--playbook', '-p',
            help="The name of the playbook to execute.\n"
                 "This is based on the playbook name in playbooks dir")
        return parser

    def run_playbook(self, playbook):
        """Executes given playbook."""
        self.ansibleManager.run_playbook(playbook, mode='create')


class NoSuchTemplateError(tobiko.TobikoException):
    message = "no such template; existing templates are: {templates}"


def main():
    """Create CLI main entry."""
    create_cmd = CreateUtil()
    create_cmd.set_stream_handler_logging_level()
    if create_cmd.args.playbook:
        create_cmd.run_playbook(create_cmd.args.playbook)


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

import tobiko
from tobiko.tests import unit


class TestCasesManagerTest(unit.TobikoUnitTest):

    test_path = os.path.dirname(__file__)

    def setUp(self):
        super(TestCasesManagerTest, self).setUp()

        top_dir = os.path.abspath(self.test_path)
        while os.path.isdir(top_dir) and top_dir != os.path.sep:
            if os.path.isdir(os.path.join(top_dir, '.stestr')):
                break
            top_dir = os.path.dirname(top_dir)
        else:
            raise self.fail("Unable to find '.stestr' directory")
        self.top_dir = top_dir
        self.repo_url = top_dir

        # Move to top directory
        original_work_dir = os.getcwd()
        os.chdir(self.top_dir)
        self.addCleanup(os.chdir, original_work_dir)

    def test_discover_testcases(self):
        testcases = tobiko.discover_testcases(test_path=self.test_path,
                                              top_dir=self.top_dir,
                                              repo_url=self.repo_url,
                                              filters=[self.id()])
        self.assertIn(self.id(), testcases)



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import logging
import sys

from tobiko.cmd import base

LOG = logging.getLogger(__name__)


class DeleteUtil(base.TobikoCMD):

    def get_parser(self):
        parser = super(DeleteUtil, self).get_parser()
        parser.add_argument(
            '--playbook', '-p',
            help="The name of the playbook to execute in delete mode.")
        return parser

    def run_playbook(self, playbook):
        """Executes given playbook."""
        self.ansibleManager.run_playbook(playbook, mode='delete')


def main():
    """Delete CLI main entry."""
    delete_cmd = DeleteUtil()
    delete_cmd.set_stream_handler_logging_level()
    delete_cmd.run_playbook(delete_cmd.args.playbook)


if __name__ == '__main__':
    sys.exit(main())



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import io
import time

from oslo_log import log

from tobiko.shell.sh import _exception
from tobiko.shell.sh import _io


LOG = log.getLogger(__name__)


class Timeout(object):

    timeout = float('inf')

    def __init__(self, timeout=None, start_time=None):
        if timeout is None:
            timeout = float('inf')
        else:
            timeout = float(timeout)
        self.timeout = timeout
        start_time = start_time and float(start_time) or time.time()
        self.start_time = start_time
        self.end_time = start_time + timeout

    def __float__(self):
        return self.timeout

    def time_left(self, now=None):
        now = now or time.time()
        return self.end_time - now

    def is_expired(self, now=None):
        raise self.time_left(now=now) <= 0.


class ShellProcess(object):

    buffer_size = io.DEFAULT_BUFFER_SIZE
    stdin = None
    stdout = None
    stderr = None
    poll_time = 0.1

    def __init__(self, command, timeout=None, stdin=None, stdout=None,
                 stderr=None, buffer_size=None, poll_time=None):
        self.command = command
        self.timeout = Timeout(timeout)
        if buffer_size is not None:
            self.buffer_size = max(64, int(buffer_size))
        if stdin:
            self.stdin = _io.ShellStdin(stdin, buffer_size=self.buffer_size)
        if stdout:
            self.stdout = _io.ShellStdout(stdout, buffer_size=self.buffer_size)
        if stderr:
            self.stderr = _io.ShellStderr(stderr, buffer_size=self.buffer_size)
        if poll_time is not None:
            self.poll_time = max(0., float(poll_time))

    def __enter__(self):
        return self

    def __exit__(self, _exception_type, _exception_value, _traceback):
        self.close()

    def close(self):
        if self.is_running:
            self.kill()
        for f in _io.select_opened_files([self.stdin,
                                          self.stdout,
                                          self.stderr]):
            f.close()

    def kill(self):
        pass

    def poll_exit_status(self):
        raise NotImplementedError

    @property
    def exit_status(self):
        return self.poll_exit_status()

    @property
    def is_running(self):
        return self.poll_exit_status() is None

    def check_is_running(self):
        exit_status = self.poll_exit_status()
        if exit_status is not None:
            raise _exception.ShellProcessTeriminated(
                command=self.command,
                exit_status=int(exit_status),
                stdin=self.stdin,
                stdout=self.stdout,
                stderr=self.stderr)

    def check_stdin_is_opened(self):
        if self.stdin.closed:
            raise _exception.ShellStdinClosed(
                command=self.command,
                stdin=self.stdin,
                stdout=self.stdout,
                stderr=self.stderr)

    def send(self, data, timeout=None):
        self.comunicate(stdin=data, timeout=timeout, wait=False)

    def wait(self, timeout=None):
        self.comunicate(stdin=None, timeout=timeout, wait=True)

    def comunicate(self, stdin=None, stdout=True, stderr=True, timeout=None,
                   wait=True):
        timeout = Timeout(timeout=timeout)
        # Avoid waiting for data in the first loop
        poll_time = 0.
        poll_files = _io.select_opened_files([stdin and self.stdin,
                                              stdout and self.stdout,
                                              stderr and self.stderr])

        while wait or stdin or poll_files:
            self.check_timeout(timeout=timeout)
            if stdin:
                self.check_is_running()
                self.check_stdin_is_opened()
            else:
                wait = wait and self.is_running

            read_ready, write_ready = _io.select_files(files=poll_files,
                                                       timeout=poll_time)
            if read_ready or write_ready:
                # Avoid waiting for data the next time
                poll_time = 0.
            else:
                # Wait for data in the following loops
                poll_time = min(self.poll_time,
                                self.check_timeout(timeout=timeout))

            if self.stdin in write_ready:
                # Write data to remote STDIN
                sent_bytes = self.stdin.write(stdin)
                if sent_bytes:
                    stdin = stdin[sent_bytes:]
                    if not stdin:
                        self.stdin.flush()
                else:
                    LOG.debug("STDIN channel closed by peer on %r", self)
                    self.stdin.close()

            if self.stdout in read_ready:
                # Read data from remote STDOUT
                chunk = self.stdout.read(self.buffer_size)
                if not chunk:
                    LOG.debug("STDOUT channel closed by peer on %r", self)
                    self.stdout.close()

            if self.stderr in read_ready:
                # Read data from remote STDERR
                chunk = self.stderr.read(self.buffer_size)
                if not chunk:
                    LOG.debug("STDERR channel closed by peer on %r", self)
                    self.stderr.close()

            poll_files = _io.select_opened_files(poll_files)

    def time_left(self, now=None, timeout=None):
        now = now or time.time()
        time_left = self.timeout.time_left(now=now)
        if timeout:
            time_left = min(time_left, timeout.time_left(now=now))
        return time_left

    def check_timeout(self, timeout=None, now=None):
        now = now or time.time()
        time_left = float('inf')
        for timeout in [self.timeout, timeout]:
            if timeout is not None:
                time_left = min(time_left, timeout.time_left(now=now))
                if time_left <= 0.:
                    ex = _exception.ShellTimeoutExpired(
                        command=self.command,
                        timeout=timeout.timeout,
                        stdin=self.stdin,
                        stdout=self.stdout,
                        stderr=self.stderr)
                    LOG.debug("%s", ex)
                    raise ex
        return time_left

    def check_exit_status(self, expected_status=0):
        exit_status = self.poll_exit_status()
        if exit_status is None:
            time_left = self.check_timeout()
            ex = _exception.ShellProcessNotTeriminated(
                command=self.command,
                time_left=time_left,
                stdin=self.stdin,
                stdout=self.stdout,
                stderr=self.stderr)
            LOG.debug("%s", ex)
            raise ex

        exit_status = int(exit_status)
        if expected_status != exit_status:
            ex = _exception.ShellCommandFailed(
                command=self.command,
                exit_status=exit_status,
                stdin=self.stdin,
                stdout=self.stdout,
                stderr=self.stderr)
            LOG.debug("%s", ex)
            raise ex

        LOG.debug("Command '%s' succeeded (exit_status=%d):\n"
                  "stdin:\n%s\n"
                  "stderr:\n%s\n"
                  "stdout:\n%s",
                  self.command, exit_status,
                  self.stdin, self.stdout, self.stderr)


def clamp(left, value, right):
    return max(left, min(value, right))



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import argparse
import logging
import sys

from tobiko.fault import executor


LOG = logging.getLogger(__name__)


class FaultCMD(object):

    def __init__(self):
        self.parser = self.get_parser()
        self.args = self.parser.parse_args()

    def get_parser(self):
        parser = argparse.ArgumentParser(add_help=True)
        parser.add_argument(
            'fault',
            help="The fault to execute (e.g. restart neutron service).\n")
        return parser

    def run(self):
        """Run faults."""
        fault_exec = executor.FaultExecutor()
        fault_exec.execute(self.args.fault)


def setup_logging(debug=None):
    """Sets the logging."""
    # pylint: disable=W0622
    format = '%(message)s'
    level = logging.DEBUG if debug else logging.INFO
    logging.basicConfig(level=level, format=format)


def main():
    """Run CLI main entry."""
    setup_logging()
    fault_cli = FaultCMD()
    fault_cli.run()


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import functools
import inspect
import unittest

import testtools

from tobiko.common import _fixture


SkipException = testtools.TestCase.skipException


def skip(reason, *args, **kwargs):
    if args or kwargs:
        reason = reason.format(*args, **kwargs)
    raise SkipException(reason)


def skip_if(reason, predicate, *args, **kwargs):
    return skip_if_match(reason, bool, predicate, *args, **kwargs)


def skip_until(reason, predicate, *args, **kwargs):
    return skip_if_match(reason, lambda x: bool(not x), predicate, *args,
                         **kwargs)


def skip_if_match(reason, match, predicate, *args, **kwargs):

    def decorator(obj):
        method = _get_decorated_method(obj)

        @functools.wraps(method)
        def wrapped_method(*_args, **_kwargs):
            return_value = predicate(*args, **kwargs)
            if match(return_value):
                skip(reason, return_value=return_value)
            return method(*_args, **_kwargs)

        if obj is method:
            return wrapped_method
        else:
            setattr(obj, method.__name__, wrapped_method)
            return obj

    return decorator


def _get_decorated_method(obj):
    if inspect.isclass(obj):
        if issubclass(obj, (unittest.TestCase, testtools.TestCase)):
            return obj.setUp
        elif _fixture.is_fixture(obj):
            return obj.setUp
        else:
            raise TypeError("Cannot decorate class {!r}".format(obj))
    else:
        if callable(obj):
            return obj
        else:
            raise TypeError("Cannot decorate object {!r}".format(obj))



# Copyright 2018 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import


class TobikoException(Exception):
    """Base Tobiko Exception.

    To use this class, inherit from it and define attribute 'message' string.
    If **properties parameters is given, then it will format message string
    using properties as key-word arguments.

    Example:

        class MyException(TobikoException):
            message = "This exception occurred because of {reason}"

        try:
            raise MyException(reason="something went wrong")
        except MyException as ex:

            # It should print:
            #   This exception occurred because of something went wrong
            print(ex)

            # It should print:
            #   something went wrong
            print(ex.reason)

    :attribute message: the message to be printed out.
    """

    message = None

    def __init__(self, message=None, **properties):
        # pylint: disable=exception-message-attribute
        message = message or self.message or "unknown reason"
        if properties:
            message = message.format(**properties)
        self.message = message
        self._properties = properties or {}
        super(TobikoException, self).__init__(message)

    def __getattr__(self, name):
        try:
            return self._properties[name]
        except KeyError:
            msg = ("{!r} object has no attribute {!r}").format(self, name)
            raise AttributeError(msg)

    def __repr__(self):
        return "{class_name}({message!r})".format(
            class_name=type(self).__name__,
            message=self.message)



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

import mock
import paramiko

import tobiko
from tobiko import config
from tobiko.shell import ssh
from tobiko.tests import unit


CONF = config.CONF


class SSHClientFixtureTest(unit.TobikoUnitTest):

    fixture = tobiko.required_fixture(ssh.SSHClientFixture)

    expected_host = None
    expected_proxy_client = None

    def setUp(self):
        super(SSHClientFixtureTest, self).setUp()
        tobiko.cleanup_fixture(self.fixture)
        self.ssh_client = mock.MagicMock(specs=paramiko.SSHClient)
        self.patch(paramiko, 'SSHClient', return_value=self.ssh_client)

    def test_init(self):
        fixture = self.fixture
        self.assertIs(self.expected_host, fixture.host)
        self.assertIs(self.expected_proxy_client, fixture.proxy_client)
        self.assertIsNone(fixture.host_config)
        self.assertIsNone(fixture.connect_parameters)

    def test_setup(self):
        fixture = self.fixture
        if not self.expected_host:
            fixture.host = 'some-host'
        fixture.username = 'some-username'
        fixture.setUp()

        ssh_config = paramiko.SSHConfig()
        for ssh_config_file in CONF.tobiko.ssh.config_files:
            if os.path.exists(ssh_config_file):
                with open(ssh_config_file) as f:
                    ssh_config.parse(f)
        expected_host_config = ssh_config.lookup(fixture.host)
        expected_host_config.pop('include', None)
        self.assertEqual(fixture.host, fixture.host_config.host)
        self.assertEqual(expected_host_config,
                         fixture.host_config.host_config)



# Copyright (c) 2019 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import inspect

from keystoneauth1 import session as keystonesession
import mock

import tobiko
from tobiko.openstack import keystone
from tobiko.tests.unit import openstack


CREDENTIALS = keystone.keystone_credentials(
    api_version=3,
    auth_url='http://127.0.0.1:4000/identiy/v3',
    username='demo',
    project_name='demo',
    password='this is a secret',
    user_domain_name='Default',
    project_domain_name='Default')


class CredentialsFixture(tobiko.SharedFixture):

    credentials = None

    def setup_fixture(self):
        self.credentials = CREDENTIALS


class DefaultCredentialsFixture(tobiko.SharedFixture):

    credentials = None

    def setup_fixture(self):
        self.credentials = DEFAULT_CREDENTIALS


DEFAULT_CREDENTIALS = keystone.keystone_credentials(
    api_version=2,
    auth_url='http://127.0.0.1:5000/identiy/v2.0',
    username='admin',
    project_name='admin',
    password='this is a secret')


class CheckSessionCredentialsMixin(object):

    def check_session_credentials(self, session, credentials):
        if credentials:
            if tobiko.is_fixture(credentials):
                self.assertIsNone(session.credentials)
                self.assertIs(credentials, session.credentials_fixture)
            else:
                self.assertIs(credentials, session.credentials)
                self.assertIsNone(session.credentials_fixture)
        else:
            self.assertIsNone(session.credentials)
            self.assertIsNone(session.credentials_fixture)


class KeystoneSessionFixtureTest(CheckSessionCredentialsMixin,
                                 openstack.OpenstackTest):

    default_credentials_fixture = (
        'tobiko.openstack.keystone._credentials.'
        'DefaultKeystoneCredentialsFixture')

    def setUp(self):
        super(KeystoneSessionFixtureTest, self).setUp()
        from tobiko.openstack.keystone import _credentials

        tobiko.remove_fixture(self.default_credentials_fixture)
        self.patch(_credentials, 'DefaultKeystoneCredentialsFixture',
                   DefaultCredentialsFixture)

    def test_init(self, credentials=None):
        session = keystone.KeystoneSessionFixture(credentials=credentials)
        self.check_session_credentials(session=session,
                                       credentials=credentials)

    def test_init_with_credentials(self):
        self.test_init(credentials=CREDENTIALS)

    def test_init_with_credentials_fixture(self):
        self.test_init(credentials=CredentialsFixture())

    def test_init_with_credentials_fixture_type(self):
        self.test_init(credentials=CredentialsFixture)

    def test_setup(self, credentials=None):
        session = keystone.KeystoneSessionFixture(credentials=credentials)
        session.setUp()
        if credentials:
            if tobiko.is_fixture(credentials):
                if inspect.isclass(credentials):
                    credentials = tobiko.get_fixture(credentials)
                self.assertIs(credentials.credentials, session.credentials)
            else:
                self.assertIs(credentials, session.credentials)
        else:
            self.assertEqual(DEFAULT_CREDENTIALS, session.credentials)

    def test_setup_with_credentials(self):
        self.test_setup(credentials=CREDENTIALS)

    def test_setup_with_credentials_fixture(self):
        self.test_setup(credentials=CredentialsFixture())

    def test_setup_with_credentials_fixture_type(self):
        self.test_setup(credentials=CredentialsFixture)


class KeystoneSessionManagerTest(CheckSessionCredentialsMixin,
                                 openstack.OpenstackTest):

    def test_init(self):
        manager = keystone.KeystoneSessionManager()

        self.assertTrue(manager)

    def test_get_session(self, credentials=None, shared=True):
        manager = keystone.KeystoneSessionManager()
        session = manager.get_session(credentials=credentials,
                                      shared=shared)
        self.assertIsInstance(session, keystone.KeystoneSessionFixture)
        self.check_session_credentials(session=session,
                                       credentials=credentials)
        if shared:
            self.assertIs(session, manager.get_session(
                credentials=credentials))
        else:
            self.assertIsNot(session, manager.get_session(
                credentials=credentials))

    def test_get_session_with_credentials(self):
        self.test_get_session(credentials=CREDENTIALS)

    def test_get_session_with_not_shared(self):
        self.test_get_session(shared=False)

    def test_get_session_with_credentials_fixture(self):
        self.test_get_session(credentials=CredentialsFixture())

    def test_get_session_with_credentials_fixture_type(self):
        self.test_get_session(credentials=CredentialsFixture)

    def test_get_session_with_init_session(self):
        mock_session = mock.MagicMock(specs=keystone.KeystoneSessionFixture)
        init_session = mock.MagicMock(return_value=mock_session)
        manager = keystone.KeystoneSessionManager()
        session = manager.get_session(credentials=CREDENTIALS,
                                      init_session=init_session)
        self.assertIs(mock_session, session)
        init_session.assert_called_once_with(credentials=CREDENTIALS)


class GetKeystomeSessionTest(openstack.OpenstackTest):

    def test_get_keystone_session(self, credentials=None, shared=True):
        session1 = keystone.get_keystone_session(credentials=credentials,
                                                 shared=shared)
        session2 = keystone.get_keystone_session(credentials=credentials,
                                                 shared=shared)
        if shared:
            self.assertIs(session1, session2)
        else:
            self.assertIsNot(session1, session2)
        self.assertIsInstance(session1, keystonesession.Session)
        self.assertIsInstance(session2, keystonesession.Session)

    def test_get_keystone_session_with_not_shared(self):
        self.test_get_keystone_session(shared=False)

    def test_get_keystone_session_with_credentials(self):
        credentials = keystone.default_keystone_credentials()
        self.test_get_keystone_session(credentials=credentials)



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from oslo_config import cfg


def register_tobiko_options(conf):

    conf.register_opts(
        group=cfg.OptGroup('ping'),
        opts=[cfg.IntOpt('count',
                         default=1,
                         help="Number of ICMP messages to wait before ending "
                              "ping command execution"),
              cfg.IntOpt('deadline',
                         default=5,
                         help="Max seconds waited from ping command before "
                              "self terminating himself"),
              cfg.StrOpt('fragmentation',
                         default=True,
                         help="If disable it will not allow ICMP messages to "
                              "be delivered in smaller fragments"),
              cfg.StrOpt('interval',
                         default=1,
                         help="Seconds of time interval between "
                              "consecutive before ICMP messages"),
              cfg.IntOpt('packet_size',
                         default=None,
                         help="Size in bytes of ICMP messages (including "
                              "headers and payload)"),
              cfg.IntOpt('timeout',
                         default=60.,
                         help="Maximum time in seconds a sequence of ICMP "
                              "messages is sent to a destination host before "
                              "reporting as a failure")])



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import os

from oslo_log import log


LOG = log.getLogger(__name__)


def makedirs(path, mode=777, exist_ok=True):
    """Creates directory and its parents if directory doesn't exists."""
    try:
        os.makedirs(path, mode)
    except os.error:
        if not exist_ok:
            raise



# Copyright (c) 2019 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import mock

from tobiko.openstack import keystone
from tobiko.tests import unit


class OpenstackTest(unit.TobikoUnitTest):

    default_keystone_credentials = keystone.keystone_credentials(
        auth_url='http://127.0.0.1:5000/identiy/v3',
        username='default',
        project_name='default',
        password='this is a secret')

    def setUp(self):
        super(OpenstackTest, self).setUp()
        from tobiko import config

        self.patch(config.CONF.tobiko, 'keystone',
                   self.default_keystone_credentials)

    def patch_get_heat_client(self, *args, **kwargs):
        from heatclient import client
        from tobiko.openstack import heat
        from tobiko.openstack.heat import _client

        kwargs.setdefault('return_value', mock.MagicMock(specs=client.Client))
        get_heat_client = self.patch(_client, 'get_heat_client', *args,
                                     **kwargs)
        self.patch(heat, 'get_heat_client', get_heat_client)
        return get_heat_client

    def patch_get_neutron_client(self, *args, **kwargs):
        from neutronclient.v2_0 import client
        from tobiko.openstack import neutron
        from tobiko.openstack.neutron import _client

        kwargs.setdefault('return_value', mock.MagicMock(specs=client.Client))
        get_neutron_client = self.patch(_client, 'get_neutron_client', *args,
                                        **kwargs)
        self.patch(neutron, 'get_neutron_client', get_neutron_client)
        return get_neutron_client



# Copyright (c) 2019 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import tobiko
from tobiko.tests import unit


def condition(value):
    return value


class PositiveSkipMethodTest(unit.TobikoUnitTest):

    @tobiko.skip_if('condition value was true',
                    condition, True)
    def test_skip_if_condition_called_with_args(self):
        self.fail('Not skipped')

    @tobiko.skip_if('condition value was true',
                    condition, value=True)
    def test_skip_if_condition_called_with_kwargs(self):
        self.fail('Not skipped')

    @tobiko.skip_until('condition value was false',
                       condition, False)
    def test_skip_until_condition_called_with_args(self):
        self.fail('Not skipped')

    @tobiko.skip_until('condition value was false',
                       condition, value=False)
    def test_skip_until_condition_called_with_kwargs(self):
        self.fail('Not skipped')


class NegativeSkipBase(unit.TobikoUnitTest):
    test_method_called = False

    def setUp(self):
        super(NegativeSkipBase, self).setUp()
        self.addCleanup(self.assert_test_method_called)

    def assert_test_method_called(self):
        self.assertTrue(self.test_method_called)


class NegativeSkipMethodTest(NegativeSkipBase):

    @tobiko.skip_if('condition value was false',
                    condition, False)
    def test_skip_if_condition_called_with_args(self):
        self.test_method_called = True

    @tobiko.skip_if('condition value was false',
                    condition, value=False)
    def test_skip_if_condition_called_with_kwargs(self):
        self.test_method_called = True

    @tobiko.skip_until('condition value was true',
                       condition, True)
    def test_skip_until_condition_called_with_args(self):
        self.test_method_called = True

    @tobiko.skip_until('condition value was true',
                       condition, value=True)
    def test_skip_until_condition_called_with_kwargs(self):
        self.test_method_called = True


@tobiko.skip_if('condition value was true',
                condition, True)
class PositiveSkipIfConditionCalledWithArgsFixture(tobiko.SharedFixture):
    pass


@tobiko.skip_if('condition value was true',
                condition, value=True)
class PositiveSkipIfConditionCalledWithKwargsFixture(tobiko.SharedFixture):
    pass


@tobiko.skip_until('condition value was false',
                   condition, False)
class PositiveSkipUntilConditionCalledWithArgsFixture(tobiko.SharedFixture):
    pass


@tobiko.skip_until('condition value was false',
                   condition, value=False)
class PositiveSkipUntilConditionCalledWithKwargsFixture(tobiko.SharedFixture):
    pass


class PositiveSkipFixtureTest(unit.TobikoUnitTest):

    def test_skip_if_condition_called_with_args(self):
        ex = self.assertRaises(
            self.skipException, tobiko.setup_fixture,
            PositiveSkipIfConditionCalledWithArgsFixture)
        self.assertEqual('condition value was true', str(ex))

    def test_skip_if_condition_called_with_kwargs(self):
        ex = self.assertRaises(
            self.skipException, tobiko.setup_fixture,
            PositiveSkipIfConditionCalledWithKwargsFixture)
        self.assertEqual('condition value was true', str(ex))

    def test_skip_until_condition_called_with_args(self):
        ex = self.assertRaises(
            self.skipException, tobiko.setup_fixture,
            PositiveSkipUntilConditionCalledWithArgsFixture)
        self.assertEqual('condition value was false', str(ex))

    def test_skip_until_condition_called_with_kwargs(self):
        ex = self.assertRaises(
            self.skipException, tobiko.setup_fixture,
            PositiveSkipUntilConditionCalledWithKwargsFixture)
        self.assertEqual('condition value was false', str(ex))


@tobiko.skip_if('condition value was false',
                condition, False)
class NegativeSkipIfConditionCalledWithArgsFixture(tobiko.SharedFixture):
    pass


@tobiko.skip_if('condition value was false',
                condition, value=False)
class NegativeSkipIfConditionCalledWithKwargsFixture(tobiko.SharedFixture):
    pass


@tobiko.skip_until('condition value was true',
                   condition, True)
class NegativeSkipUntilConditionCalledWithArgsFixture(tobiko.SharedFixture):
    pass


@tobiko.skip_until('condition value was true',
                   condition, value=True)
class NegativeSkipUntilConditionCalledWithKwargsFixture(tobiko.SharedFixture):
    pass


class NegativeSkipFixtureTest(unit.TobikoUnitTest):

    def test_skip_if_condition_called_with_args(self):
        fixture = tobiko.setup_fixture(
            NegativeSkipIfConditionCalledWithArgsFixture)
        self.assertIsInstance(
            fixture, NegativeSkipIfConditionCalledWithArgsFixture)

    def test_skip_if_condition_called_with_kwargs(self):
        fixture = tobiko.setup_fixture(
            NegativeSkipIfConditionCalledWithKwargsFixture)
        self.assertIsInstance(
            fixture, NegativeSkipIfConditionCalledWithKwargsFixture)

    def test_skip_until_condition_called_with_args(self):
        fixture = tobiko.setup_fixture(
            NegativeSkipUntilConditionCalledWithArgsFixture)
        self.assertIsInstance(
            fixture, NegativeSkipUntilConditionCalledWithArgsFixture)

    def test_skip_until_condition_called_with_kwargs(self):
        fixture = tobiko.setup_fixture(
            NegativeSkipUntilConditionCalledWithKwargsFixture)
        self.assertIsInstance(
            fixture, NegativeSkipUntilConditionCalledWithKwargsFixture)


@tobiko.skip_if('condition value was true',
                condition, True)
class PositiveSkipIfConditionCalledWithArgsTest(unit.TobikoUnitTest):

    def test_fail(self):
        self.fail('Not skipped')


@tobiko.skip_if('condition value was true',
                condition, value=True)
class PositiveSkipIfConditionCalledWithKwargsTest(unit.TobikoUnitTest):

    def test_fail(self):
        self.fail('Not skipped')


@tobiko.skip_until('condition value was false',
                   condition, False)
class PositiveSkipUntilConditionCalledWithArgsTest(unit.TobikoUnitTest):

    def test_fail(self):
        self.fail('Not skipped')


@tobiko.skip_until('condition value was false',
                   condition, value=False)
class PositiveSkipUntilConditionCalledWithKwargsTest(unit.TobikoUnitTest):

    def test_fail(self):
        self.fail('Not skipped')


@tobiko.skip_if('condition value was true',
                condition, False)
class NegativeSkipIfConditionCalledWithArgsTest(NegativeSkipBase):

    def test_fail(self):
        self.test_method_called = True


@tobiko.skip_if('condition value was true',
                condition, value=False)
class NegativeSkipIfConditionCalledWithKwargsTest(NegativeSkipBase):

    def test_fail(self):
        self.test_method_called = True


@tobiko.skip_until('condition value was false',
                   condition, True)
class NegativeSkipUntilConditionCalledWithArgsTest(NegativeSkipBase):

    def test_fail(self):
        self.test_method_called = True


@tobiko.skip_until('condition value was false',
                   condition, value=True)
class NegativeSkipUntilConditionCalledWithKwargsTest(NegativeSkipBase):

    def test_fail(self):
        self.test_method_called = True



# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
BASE_DIR = os.path.dirname(os.path.abspath(__file__))
TOBIKO_DIR = os.path.abspath(os.path.join(BASE_DIR, "..", ".."))
sys.path.insert(0, TOBIKO_DIR)


# -- Project information -----------------------------------------------------

project = 'Tobiko'
copyright = "2019, Red Hat"
author = "Tobiko's Team"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# Version info
from tobiko import version
release = version.release
# The short X.Y version.
version = version.version


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx.ext.ifconfig',
    'sphinx.ext.graphviz',
    'sphinx.ext.todo',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = []

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# openstackdocstheme options
repository_name = 'x/tobiko'
bug_project = 'tobiko'
bug_tag = 'doc'

# Set to True if using StoryBoard
use_storyboard = True

todo_include_todos = True


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
    "canonical_url": "https://docs.openstack.org/tobiko/latest/",
    "logo_only": False,
    "display_version": True,
    "prev_next_buttons_location": "top",
    "style_external_links": True,
    # Toc options
    "collapse_navigation": True,
    "sticky_navigation": True,
    "navigation_depth": 4,
    "includehidden": True,
    "titles_only": False,
}


# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']



# Copyright (c) 2019 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from tobiko.shell.sh import _command
from tobiko.shell.sh import _exception
from tobiko.shell.sh import _execute


ShellError = _exception.ShellError
ShellCommandFailed = _exception.ShellCommandFailed
ShellTimeoutExpired = _exception.ShellTimeoutExpired
ShellProcessTeriminated = _exception.ShellProcessTeriminated
ShellProcessNotTeriminated = _exception.ShellProcessNotTeriminated
ShellStdinClosed = _exception.ShellStdinClosed

execute = _execute.execute
local_execute = _execute.local_execute
ssh_execute = _execute.ssh_execute

shell_command = _command.shell_command
